/*============================================================================*/
/* eCall ANSI C fixed-point reference source code                             */
/*                                                                            */
/* File:         standalone.h                                                 */
/* Version:      8.6.0 (Rel8) / 9.4.0 (Rel9)                                  */
/* Date:         2011-02-08                                                   */
/* Description:  eCall standalone wrapper (header file)                       */
/*----------------------------------------------------------------------------*/

#ifndef STANDALONE_H_
#define STANDALONE_H_

#define MAX_FRAMES 2500
#define NUM_MODES  6

#include <stdio.h>
#include "ecall_defines.h"
#include "ecall_control.h"


/*============================================================================*/
/* Variables                                                                  */
/*----------------------------------------------------------------------------*/

typedef enum {
  Udef = -1, Ivs, IvsRx, IvsTx, Psap, PsapRx, PsapTx
} RunMode;

typedef struct {
  int    count;
  char   filename[501];
  fpos_t filepos;
} PortLog;

typedef struct {
  RunMode mode;
  PortLog prxlog;
  PortLog ptxlog;

  Bool ivsPush;
  Bool msdReceived;

  int indexCase;
  int frameCount;
  int frameStart;
  int frameStartHlack;

  Ord8  msd[MSD_MAX_LENGTH];
  Int16 pcmBuffer[MAX_FRAMES*PCM_LENGTH];

  char *campaign;
  char *inbase;
  char *outbase;
  char *portbase;
} EcallShell;

/*============================================================================*/
/* Functions                                                                  */
/*----------------------------------------------------------------------------*/

void Parse(int, char**);
void PrintUsage(const char*);
void ConvertMsd(const char*, Ord8*);

void EcallReset(int);
void EcallProcess(void);

int GetBuffer(const char*, const char*, int, int);
int PutBuffer(const char*, const char*, int, int);

void PortLogInit(PortLog*, const char*);

int PortLogWrite(PortLog*, CtrlPort*);
int PortLogRead(PortLog*, CtrlPort*);

#endif
