/**
 * Connects IVS to CONTROL via IPC.
 *
 * @copyright 2008, Research In Motion Ltd.
 *
 * @version $Id: //depot/main/dsp/ecall/c/testlab/ivs.c#9 $
 */

#include <stdlib.h>

#include "rimtypes.h"
#include "error.h"
#include "msg.h"
#include "ipc.h"
#include "testlab.h"
#include "modem.h"

static IpcConn *conn;

BOOL IvsRun(void) {
  Msg msg;

  if (IpcRecv(conn, &msg) == NULL) return FALSE;

  if (msg.id == MsgIvsReset) {
    LogInfo("Received MsgIvsReset");
    if (msg.u.ivsReset.n <= MSD_MAX_LENGTH) {
      IvsReset(msg.u.ivsReset.msd, msg.u.ivsReset.n);
      msg.id = MsgReady;
      IpcSend(conn, &msg);
    }
    else {
      sprintf(msg.u.error.msg, "MSD length too big %d >= %d",
        msg.u.ivsReset.n, MSD_MAX_LENGTH);
      msg.id = MsgError;
      IpcSend(conn, &msg);
    }
  }
  else if (msg.id == MsgPcmData) {
    IvsProcess(msg.u.pcmData.pcm);
    IpcSend(conn, &msg);
  }
  else if (msg.id == MsgIvsTx) {
    LogInfo("Received MsgIvsStart");
    IvsSendStart();
  }
  else {
    sprintf(msg.u.error.msg, "Unknown msg type %d", msg.id);
    msg.id = MsgError;
    IpcSend(conn, &msg);
  }

  return TRUE;
}

void IvsInit(const char *host, const char *port) {
  LogInfo("Starting IVS");
  conn = IpcConnect(host, port);
  IpcRegisterTask(conn, IvsRun);
}

void IvsReceiveAck(void) {
  Msg msg;
  msg.id = MsgIvsAck;
  IpcSend(conn, &msg);
}

void IvsReceiveHlack(const Ord8 data) {
  Msg msg;
  msg.id = MsgIvsHlack;
  msg.u.ivsHlack.data = data;
  IpcSend(conn, &msg);
}

void IvsCatchEvent(int ie) {
  switch (ie) {
    /* custom event handlers to be implemented here... */
    default:
      break;
  }
}
