/*============================================================================*/
/* eCall ANSI C fixed-point reference source code                             */
/*                                                                            */
/* File:         ecall_rom.c                                                  */
/* Version:      8.6.0 (Rel8) / 9.4.0 (Rel9)                                  */
/* Date:         2011-02-08                                                   */
/* Description:  ROM data arrays                                              */
/*----------------------------------------------------------------------------*/

#include "ecall_defines.h"
#include "ecall_rom.h"


/*============================================================================*/
/* Synchronization                                                            */
/*----------------------------------------------------------------------------*/

const Int16 wakeupSin500[16] =
{
  0, 7654,  14142,  18478,  20000,  18478,  14142,  7654,
  0, -7654, -14142, -18478, -20000, -18478, -14142, -7654,
};

const Int16 wakeupCos500[16] =
{
  20000,  18478,  14142,  7654,  0, -7654, -14142, -18478,
  -20000, -18478, -14142, -7654, 0, 7654,  14142,  18478,
};

const Int16 wakeupSin800[10] =
{
  0, 11756,  19021,  19021,  11756,
  0, -11756, -19021, -19021, -11756
};

const Int16 wakeupCos800[10] =
{
  20000,  16180,  6180,  -6180, -16180,
  -20000, -16180, -6180, 6180,  16180
};

/*----------------------------------------------------------------------------*/

const Int16 syncPulseForm[5] =
{
  1515, 485, -1327, -95, -482
};

const Int16 syncSequence[15] =
{
  1, 1, 1, 1, -1, 1, -1, 1, 1, -1, -1, 1, -1, -1, -1
};

const Int16 syncIndexPreamble[SYNC_IDXLEN] =
{
  0,   1,   2,   3,   4,
  22,  23,  24,  25,  26,
  44,  45,  46,  47,  48,
  66,  67,  68,  69,  70,
  88,  89,  90,  91,  92,
  110, 111, 112, 113, 114,
  132, 133, 134, 135, 136,
  154, 155, 156, 157, 158,
  176, 177, 178, 179, 180,
  198, 199, 200, 201, 202,
  220, 221, 222, 223, 224,
  242, 243, 244, 245, 246,
  264, 265, 266, 267, 268,
  286, 287, 288, 289, 290,
  308, 309, 310, 311, 312
};

const Int16 syncFrame[10*PCM_LENGTH] =
{
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      20000,  0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      -20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      -20000, 0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      -20000, 0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      -20000, 0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      20000,  0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      20000,  0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      20000
};

/*============================================================================*/
/* Uplink format                                                              */
/*----------------------------------------------------------------------------*/

const Int16 indexBits[24] =
{
  0, 0, 0,
  0, 0, 1,
  0, 1, 0,
  0, 1, 1,
  1, 0, 0,
  1, 0, 1,
  1, 1, 0,
  1, 1, 1
};

/*----------------------------------------------------------------------------*/

const Int16 m4smp_ulPulse[16] =
{
  0,     0,     0,     40,    -200,  560,  -991,   -1400,
  7636,  15000, 7636,  -1400, -991,  560,  -200,   40
};

const Int16 m8smp_ulPulse[32] =
{
  0,     0,     0,     0,     0,     40,    -200,  560,
  -991,  -1400, 7636,  15000, 7636,  -1400, -991,  560,
  -200,  40,    0,     0,     0,     0,     0,     0,
  0,     0,     0,     0,     0,     0,     0,     0
};

const Int16 m4smp_ulPulseMatch[64] =
{
  0,     0,     0,     23,    -113,  279,   -387,  -1336,
  4892,  11262, 3665,  -7236, -6829, -3000, -2790, -2050,
  -599,  147,   513,   -274,  282,   -8,    6,     111,
  -182,  349,   -446,  -1309, 4858,  11260, 3647,  -7250,
  4625,  10568, 3481,  -7619, -6964, -3158, -2870, -2083,
  -1047, -695,  -130,  176,   263,   826,   205,   -685,
  -120,  300,   -385,  -1329, 4897,  11264, 3668,  -7237,
  -6826, -3000, -2790, -2052, -1023, -723,  -146,  129
};

const Int16 m8smp_ulPulseMatch[128] =
{
  0,     0,     0,     0,     0,     23,    -113,  279,
  -387,  -1336, 4892,  11262, 3665,  -7236, -6829, -3000,
  -2790, -2050, -1022, -722,  -145,  105,   339,   499,
  548,   609,   576,   563,   500,   448,   381,   317,
  0,     0,     0,     0,     0,     0,     0,     0,
  0,     0,     0,     0,     0,     23,    -113,  279,
  -387,  -1336, 4892,  11262, 3665,  -7236, -6829, -3000,
  -2790, -2050, -1022, -722,  -145,  105,   339,   499,
  -120,  -15,   113,   -14,   36,    15,    1,     16,
  -8,    8,     -7,    2,     -4,    -2,    -2,    -3,
  -2,    -2,    -1,    -2,    -1,    22,    -114,  278,
  -387,  -1337, 4892,  11262, 3665,  -7236, -6828, -2999,
  -599,  -1039, 4880,  11349, 3725,  -7236, -6770, -3020,
  -2760, -2068, -1016, -733,  -152,  97,    329,   492,
  539,   603,   569,   558,   496,   445,   378,   315,
  256,   199,   151,   107,   72,    67,    -91,   283
};

/*----------------------------------------------------------------------------*/

const Int16 m4smp_mgTable[66] =
{
  -1,
  0,  1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12, 13, 14,
  -2, -3, -4, -5, -1, -6,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
  -2, -3, -4, -5, -1, -6,
  30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45,
  -2, -3, -4, -5, -1, -6, -1
};

const Int16 m8smp_mgTable[116] =
{
  -1,
  0,  1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12, 13, 14,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
  -2, -3, -4, -5, -1, -6, -1, -1,
  30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
  45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
  -2, -3, -4, -5, -1, -6, -1, -1,
  60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75,
  76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91,
  -2, -3, -4, -5, -1, -6, -1
};

/*============================================================================*/
/* Downlink format                                                            */
/*----------------------------------------------------------------------------*/

const Int16 dlPcmData[4][NRF_DLDATA*PCM_LENGTH] =
{
  {
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636
  },
  {
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636
  },
  {
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0
  },
  {
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  1400,   991,    -560,   200,    -40,    0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    200,    -560,   991,    1400,   -7636,  -15000, -7636,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  40,     -200,   560,    -991,   -1400,  7636,   15000,  7636,
  -1400,  -991,   560,    -200,   40,     0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -1400,  7636,   15000,  7636,   -1400,  -991,   560,    -200,
  40,     0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      40,     -200,   560,    -991,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  -40,    0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      -40,    200,    -560,   991,
  1400,   -7636,  -15000, -7636,  1400,   991,    -560,   200
  }
};

/*----------------------------------------------------------------------------*/

const Int16 dlPcmDataMatch[4][NRF_DLDATA*PCM_LENGTH] =
{
  {
  -24,    -8,     21,     2,      8,      5,      1,      4,
  -1,     2,      -1,     0,      -1,     -1,     -1,     -1,
  -1,     -1,     -1,     -1,     -25,    113,    -280,   387,
  1338,   -4894,  -11264, -3665,  7236,   6829,   3000,   2791,
  1227,   5385,   12014,  3680,   -7464,  -7150,  -3613,  -3296,
  -2720,  -1568,  -1299,  -628,   -331,   -29,    200,    302,
  426,    433,    467,    435,    412,    365,    319,    270,
  220,    177,    135,    101,    94,     -68,    304,    -378,
  -492,   529,    -65,    111,    93,     -44,    93,     -67,
  68,     -161,   282,    -415,   -1355,  4877,   11242,  3653,
  -7254,  -6838,  -3011,  -2798,  -2058,  -1027,  -726,   -148,
  105,    340,    500,    550,    611,    578,    565,    502,
  425,    495,    39,     646,    1539,   -4741,  -11155, -3591,
  7281,   6851,   3004,   2782,   2034,   1000,   697,    119,
  -132,   -364,   -521,   -569,   -626,   -590,   -575,   -509,
  -455,   -386,   -321,   -259,   -201,   -152,   -108,   -73,
  -43,    -20,    -2,     10,     19,     24,     27,     27,
  51,     -89,    301,    -368,   -1322,  4907,   11274,  3673,
  -7230,  -6825,  -2997,  -2789,  -2051,  -1023,  -723,   -147,
  105,    339,    498,    549,    609,    576,    563,    500,
  1296,   1253,   -573,   -309,   205,    -291,   70,     -62,
  -17,    31,     -26,    48,     -8,     38,     8,      24,
  15,     15,     14,     9,      10,     6,      6,      3,
  -21,    115,    -278,   387,    1338,   -4894,  -11264, -3666,
  7236,   1602,   -7423,  -178,   9396,   8271,   3873,   3023,
  2065,   639,    267,    -480,   -736,   -990,   -1114,  -1110,
  -1115,  -1005,  -929,   -794,   -683,   -558,   -447,   -346,
  -255,   -181,   -116,   -67,    -51,    116,    -255,   425,
  536,    -489,   99,     -82,    -69,    63,     -78,    77,
  -36,    53,     0,      29,     17,     16,     19,     10,
  39,     -107,   288,    -382,   -1333,  4896,   11265,  3666,
  -7236,  -6829,  -3001,  -2792,  -2052,  -1024,  -724,   -147,
  106,    340,    499,    549,    634,    463,    843,    114,
  -889,   5275,   11582,  3923,   -7036,  -6677,  -2892,  -2717,
  -2007,  -1002,  -719,   -155,   89,     317,    474,    523,
  584,    553,    542,    482,    433,    368,    307,    250,
  170,    262,    -174,   459,    1381,   -4872,  -11259, -3673,
  7220,   6807,   2976,   2766,   2027,   1000,   702,    128,
  -122,   -353,   -510,   -558,   -616,   -581,   -567,   -502,
  -449,   -381,   -318,   -257,   -200,   -151,   -107,   -73,
  -67,    -28,    18,     11,     26,     28,     27,     31,
  25,     26,     21,     19,     15,     13,     10,     7,
  5,      3,      2,      1,      -24,    113,    -280,   386,
  1337,   -4895,  -11265, -3666,  7235,   6828,   2999,   2790,
  2075,   1030,   701,    144,    -114,   -345,   -501,   -553,
  -609,   -579,   -563,   -501,   -448,   -380,   -317,   -257,
  -224,   -38,    -387,   314,    1295,   -4914,  -11267, -3656,
  7254,   6852,   3026,   2818,   2077,   1047,   744,    165,
  -90,    -327,   -489,   -542,   -604,   -573,   -561,   -499,
  -448,   -381,   -319,   -259,   -177,   -267,   170,    -462,
  -1383,  4871,   11259,  3674,   -7219,  -6807,  -2975,  -2765,
  -2026,  -999,   -701,   -127,   122,    354,    510,    558,
  -232,   4936,   11879,  4038,   -6901,  -6424,  -2795,  -2485,
  -1911,  -837,   -629,   -55,    159,    372,    520,    549,
  607,    562,    549,    481,    430,    362,    300,    242,
  186,    141,    98,     66,     62,     -97,    280,    -398,
  -1357,  -357,   816,    670,    84,     396,    130,    69,
  105,    -56,    34,     -83,    -25,    -76,    -53,    -61,
  -56,    -47,    -46,    -34,    -32,    -23,    -19,    -13,
  14,     -120,   276,    -389,   -1338,  4895,   11266,  3668
  },
  {
  0,      0,      0,      0,      0,      0,      0,      0,
  0,      0,      0,      0,      0,      0,      0,      0,
  24,     -113,   279,    -387,   -1338,  4894,   11263,  3665,
  -7236,  -6829,  -3000,  -2791,  -2051,  -1023,  -723,   -146,
  106,    340,    500,    550,    610,    577,    564,    501,
  449,    381,    318,    258,    176,    266,    -171,   461,
  1382,   -4872,  -11260, -3674,  7219,   6806,   2974,   2764,
  2026,   999,    701,    127,    -122,   -354,   -510,   -558,
  -616,   -581,   -567,   -503,   -449,   -381,   -318,   -257,
  -200,   -151,   -107,   -73,    -43,    -20,    -3,     10,
  42,     -90,    305,    -360,   -1312,  4918,   11285,  3684,
  -7220,  -6816,  -2990,  -2783,  -2045,  -1019,  -720,   -145,
  107,    340,    499,    549,    633,    462,    842,    113,
  -890,   5274,   11581,  3922,   -7036,  -6677,  -2892,  -2718,
  -2008,  -1002,  -719,   -155,   89,     317,    474,    523,
  584,    553,    542,    482,    433,    368,    307,    250,
  170,    262,    -174,   459,    1381,   -4872,  -11259, -3673,
  7220,   6807,   2976,   2766,   2027,   1000,   702,    128,
  -122,   -353,   -510,   -558,   -616,   -581,   -567,   -502,
  -449,   -381,   -318,   -257,   -200,   -151,   -107,   -73,
  -43,    -20,    -3,     10,     18,     24,     26,     27,
  2,      138,    -257,   406,    1354,   -4880,  -11253, -3657,
  7243,   6833,   3003,   2793,   2052,   1023,   722,    145,
  -107,   -342,   -501,   -551,   -611,   -578,   -565,   -502,
  -1297,  -1254,  572,    308,    -205,   291,    -70,    62,
  17,     -31,    26,     -48,    9,      -38,    -8,     -24,
  -15,    -15,    -14,    -9,     -10,    -6,     -6,     -3,
  21,     -115,   278,    -387,   -1338,  4894,   11264,  3666,
  -7236,  -6828,  -2999,  -2790,  -2051,  -1022,  -722,   -146,
  107,    341,    500,    550,    610,    577,    564,    501,
  449,    381,    318,    258,    201,    152,    108,    74,
  68,     -92,    283,    -396,   -1356,  4870,   11238,  3639,
  -6414,  -5981,  -3912,  -3377,  -2087,  -1366,  -1051,  97,
  1378,   -4548,  -10796, -3060,  7855,   7467,   3598,   3343,
  2542,   1444,   1078,   433,    122,    -168,   -374,   -464,
  -557,   -549,   -556,   -508,   -466,   -405,   -345,   -286,
  -228,   -179,   -132,   -95,    -62,    -36,    -16,    0,
  -14,    131,    -257,   412,    1363,   -4870,  -11241, -3645,
  7254,   6844,   3012,   2800,   2059,   1028,   726,    148,
  -105,   -340,   -500,   -550,   -611,   -578,   -565,   -502,
  -450,   -382,   -319,   -259,   -201,   -153,   -109,   -74,
  -20,    -135,   276,    -378,   -1320,  4917,   11289,  3692,
  -7211,  -6805,  -2978,  -2772,  -2035,  -1010,  -712,   -138,
  112,    345,    502,    551,    611,    577,    563,    500,
  -401,   4735,   11629,  3792,   -7150,  -6654,  -3005,  -2669,
  -2068,  -968,   -733,   -136,   99,     329,    491,    532,
  600,    561,    553,    489,    440,    373,    311,    253,
  196,    150,    106,    72,     67,     -92,    283,    -396,
  -507,   516,    -74,    104,    112,    -161,   370,    -455,
  -1294,  4846,   11267,  3638,   -7253,  -6845,  -3020,  -2802,
  -2067,  -1031,  -733,   -152,   100,    337,    497,    548,
  609,    576,    564,    501,    449,    382,    319,    259,
  -647,   4508,   11422,  3609,   -7306,  -6784,  -3108,  -2751,
  -2128,  -1012,  -762,   -154,   91,     328,    495,    540,
  609,    572,    564,    499,    449,    382,    319,    259,
  202,    154,    110,    75,     69,     -91,    284,    -396,
  341,    1388,   -965,   -463,   92,     -491,   52,     -204,
  -17,    -38,    -26,    30,     -7,     45,     14,     40,
  24,     31,     25,     22,     21,     15,     14,     10,
  -16,    119,    -275,   389,    1339,   -4893,  -11264, -3666
  },
  {
  848,    872,    -890,   -567,   5,      -443,   -38,    -136,
  -60,    10,     -30,    57,     9,      61,     33,     51,
  40,     39,     36,     28,     26,     19,     17,     11,
  -15,    119,    -276,   389,    1338,   -4894,  -11265, -3667,
  5538,   10310,  15201,  6891,   -5305,  -5340,  -2352,  -2461,
  -2158,  -1339,  -1207,  -734,   -503,   -286,   -81,    7,
  136,    165,    221,    222,    229,    215,    197,    176,
  150,    127,    102,    81,     85,     -69,    309,    -368,
  -504,   534,    -30,    125,    113,    -29,    103,    -55,
  49,     -41,    5,      -25,    -16,    -16,    -21,    -13,
  -18,    -10,    -12,    -8,     -32,    108,    -283,   384,
  1336,   -4895,  -11264, -3665,  7237,   6830,   3001,   2792,
  2925,   -3323,  -10610, -3390,  7236,   6461,   2612,   2189,
  1502,   410,    174,    -374,   -564,   -732,   -834,   -816,
  -825,   -737,   -683,   -581,   -499,   -407,   -325,   -252,
  -185,   -131,   -83,    -47,    -42,    117,    -261,   416,
  524,    -502,   86,     -95,    -81,    53,     -87,    70,
  -43,    48,     -4,     26,     -9,     128,    -261,   397,
  1353,   -4886,  -11254, -3660,  7242,   6833,   3002,   2793,
  2052,   1023,   722,    146,    -107,   -341,   -500,   -551,
  -611,   -578,   -564,   -501,   -449,   -382,   -318,   -258,
  -201,   -153,   -109,   -74,    -44,    -21,    -4,     9,
  42,     -90,    305,    -360,   -1312,  4918,   11285,  3684,
  -7220,  -6816,  -2990,  -2783,  -2045,  -1019,  -720,   -144,
  107,    340,    499,    549,    633,    462,    842,    113,
  -890,   5274,   11581,  3922,   -7036,  -6677,  -2892,  -2718,
  -2008,  -1002,  -719,   -155,   89,     317,    474,    523,
  584,    553,    542,    482,    433,    368,    307,    250,
  170,    262,    -174,   459,    1381,   -4872,  -11259, -3673,
  7220,   6807,   2976,   2766,   2027,   1000,   702,    128,
  -122,   -353,   -510,   -558,   -616,   -581,   -567,   -502,
  -449,   -381,   -318,   -257,   -200,   -151,   -107,   -73,
  -67,    -28,    18,     11,     26,     28,     27,     31,
  25,     26,     21,     19,     15,     13,     10,     7,
  5,      3,      2,      1,      -24,    113,    -280,   386,
  1337,   -4895,  -11265, -3666,  7235,   6828,   2999,   2790,
  2075,   1030,   701,    144,    -114,   -345,   -501,   -553,
  -609,   -579,   -563,   -501,   -448,   -380,   -317,   -257,
  -224,   -38,    -387,   314,    1295,   -4914,  -11267, -3656,
  7254,   6852,   3026,   2818,   2077,   1047,   744,    165,
  -90,    -327,   -489,   -542,   -604,   -573,   -561,   -499,
  -448,   -381,   -319,   -259,   -177,   -267,   170,    -462,
  -1383,  4871,   11259,  3674,   -7219,  -6807,  -2975,  -2765,
  -2026,  -999,   -701,   -127,   122,    354,    510,    558,
  -232,   4936,   11879,  4038,   -6901,  -6424,  -2795,  -2485,
  -1911,  -837,   -629,   -55,    159,    372,    520,    549,
  607,    562,    549,    481,    430,    362,    300,    242,
  186,    141,    98,     66,     62,     -97,    280,    -398,
  -1357,  -357,   816,    670,    84,     396,    130,    69,
  105,    -56,    34,     -83,    -25,    -76,    -53,    -61,
  -56,    -47,    -46,    -34,    -32,    -23,    -19,    -13,
  14,     -120,   276,    -389,   -1338,  4895,   11266,  3668,
  -5537,  -5083,  -4778,  -3922,  -2040,  -1908,  -798,   -416,
  -13,    360,    441,    664,    629,    698,    631,    602,
  530,    459,    390,    314,    253,    190,    142,    97,
  38,     146,    -268,   382,    1321,   -4918,  -11292, -3695,
  6359,   5929,   3865,   3335,   2027,   1450,   748,    271,
  -54,    -356,   -474,   -609,   -596,   -751,   -318,   -938,
  -1826,  4475,   10911,  3381,   -7462,  -6999,  -3125,  -2875,
  -2104,  -1050,  -730,   -139,   124,    364,    527,    578
  },
  {
  0,      0,      0,      0,      0,      0,      0,      0,
  -24,    113,    -279,   387,    1338,   -4894,  -11263, -3665,
  7236,   6829,   3000,   2791,   2051,   1023,   723,    146,
  -106,   -340,   -500,   -550,   -610,   -577,   -564,   -501,
  -449,   -381,   -318,   -258,   -201,   -153,   -108,   -74,
  -44,    -21,    -4,     9,      18,     23,     26,     27,
  2,      138,    -258,   406,    1354,   -4880,  -11253, -3657,
  7243,   6834,   3003,   2793,   2052,   1023,   722,    145,
  -107,   -342,   -501,   -551,   -635,   -465,   -844,   -115,
  889,    -5275,  -11582, -3923,  7036,   6677,   2892,   2717,
  2008,   1002,   719,    155,    -89,    -317,   -474,   -523,
  -584,   -553,   -542,   -482,   -433,   -368,   -307,   -250,
  654,    724,    -996,   -639,   -39,    -465,   -43,    -128,
  -44,    32,     -5,     82,     34,     84,     54,     69,
  56,     52,     46,     36,     32,     23,     20,     13,
  -15,    119,    -276,   388,    1337,   -4895,  -11266, -3668,
  6385,   5954,   3888,   3355,   2045,   1464,   760,    280,
  -47,    -351,   -471,   -607,   -595,   -751,   -318,   -939,
  -1827,  4474,   10910,  3379,   -7463,  -7000,  -3125,  -2876,
  -2104,  -1050,  -730,   -139,   124,    364,    527,    578,
  1486,   1475,   -303,   -45,    471,    -47,    292,    131,
  147,    167,    82,     132,    54,     82,     37,     41,
  22,     14,     9,      0,      0,      -6,     -6,     -8,
  -32,    105,    -287,   380,    1332,   -4898,  -11268, -3668,
  5537,   10310,  15202,  6892,   -5304,  -5339,  -2351,  -2460,
  -2157,  -1338,  -1206,  -733,   -502,   -286,   -80,    8,
  136,    165,    221,    222,    229,    215,    197,    176,
  149,    127,    102,    81,     85,     -69,    309,    -368,
  368,    1413,   -942,   -443,   110,    -477,   63,     -195,
  -10,    -33,    -23,    32,     -6,     45,     13,     39,
  23,     29,     24,     21,     20,     14,     13,     9,
  -17,    118,    -276,   389,    1339,   -4893,  -11264, -3666,
  6387,   5956,   3889,   3356,   2046,   1465,   760,    281,
  -47,    -350,   -470,   -607,   -619,   -638,   -597,   -552,
  -465,   -533,   -75,    -673,   -1565,  4722,   11138,  3580,
  -7289,  -6856,  -3007,  -2784,  -2034,  -999,   -696,   -118,
  -714,   -506,   1414,   1137,   623,    1035,   614,    646,
  516,    377,    351,    203,    192,    92,     75,     22,
  2,      -19,    -34,    -38,    -45,    -43,    -43,    -39,
  -11,    -144,   254,    -408,   -1355,  4881,   11254,  3659,
  -6392,  -5959,  -3891,  -3357,  -2021,  -1578,  -480,   -667,
  -1290,  5245,   11735,  4273,   -6616,  -6190,  -2402,  -2239,
  -1562,  -603,   -369,   140,    333,    512,    625,    635,
  663,    604,    571,    494,    431,    358,    291,    230,
  197,    13,     364,    -334,   -1312,  4900,   11255,  3647,
  -7261,  -6857,  -3029,  -2820,  -2078,  -1047,  -744,   -164,
  91,     328,    490,    543,    605,    574,    562,    500,
  449,    382,    319,    259,    202,    154,    110,    75,
  -803,   4377,   11317,  3527,   -7367,  -6828,  -3138,  -2768,
  -2137,  -1013,  -758,   -146,   100,    339,    506,    551,
  619,    581,    572,    506,    455,    386,    322,    262,
  203,    155,    110,    75,     69,     -92,    283,    -396,
  -507,   516,    -75,    103,    63,     65,     -189,   319,
  1381,   -4941,  -11260, -3692,  7220,   6813,   2980,   2780,
  2035,   1015,   712,    140,    -112,   -344,   -502,   -552,
  -611,   -578,   -564,   -501,   -448,   -381,   -317,   -257,
  -224,   -160,   -87,    -72,    -36,    -16,    -2,     13,
  17,     25,     25,     27,     25,     23,     21,     18,
  15,     12,     10,     8,      -19,    117,    -277,   388,
  1338,   -4894,  -11264, -3666,  7235,   6828,   2999,   2790
  }
};

/*============================================================================*/
/* FEC encoder/decoder                                                        */
/*----------------------------------------------------------------------------*/

const Ord1 crcPolynomial[NRB_CRC+1] =
{
  1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1,
  0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1
};

const Ord16 stateTrans[16] =
{
  0, 1, 2, 3, 5, 4, 7, 6, 1, 0, 3, 2, 4, 5, 6, 7
};

const Ord16 revStateTrans[16] =
{
  0, 4, 4, 0, 1, 5, 5, 1, 6, 2, 2, 6, 7, 3, 3, 7
};

const Ord1 outputParity[16] =
{
  0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1
};

/*----------------------------------------------------------------------------*/

const Int16 logExpTable[LOGEXP_RES] =
{
  177, 175, 173, 172, 170, 168, 166, 164, 162, 160,
  158, 156, 155, 153, 151, 149, 147, 146, 144, 142,
  141, 139, 137, 136, 134, 132, 131, 129, 128, 126,
  124, 123, 121, 120, 118, 117, 115, 114, 113, 111,
  110, 108, 107, 106, 104, 103, 102, 100, 99,  98,
  96,  95,  94,  93,  92,  90,  89,  88,  87,  86,
  85,  83,  82,  81,  80,  79,  78,  77,  76,  75,
  74,  73,  72,  71,  70,  69,  68,  67,  66,  65,
  64,  64,  63,  62,  61,  60,  59,  59,  58,  57,
  56,  55,  55,  54,  53,  52,  52,  51,  50,  49,
  49,  48,  47,  47,  46,  45,  45,  44,  43,  43,
  42,  42,  41,  40,  40,  39,  39,  38,  38,  37,
  37,  36,  35,  35,  34,  34,  33,  33,  32,  32,
  32,  31,  31,  30,  30,  29,  29,  28,  28,  28,
  27,  27,  26,  26,  26,  25,  25,  25,  24,  24,
  23,  23,  23,  22,  22,  22,  21,  21,  21,  21,
  20,  20,  20,  19,  19,  19,  18,  18,  18,  18,
  17,  17,  17,  17,  16,  16,  16,  16,  15,  15,
  15,  15,  14,  14,  14,  14,  14,  13,  13,  13,
  13,  13,  12,  12,  12,  12,  12,  12,  11,  11,
  11,  11,  11,  11,  10,  10,  10,  10,  10,  10,
  9,   9,   9,   9,   9,   9,   9,   8,   8,   8,
  8,   8,   8,   8,   8,   7,   7,   7,   7,   7,
  7,   7,   7,   7,   7,   6,   6,   6,   6,   6,
  6,   6,   6,   6,   6,   6,   5,   5,   5,   5,
  5,   5,   5,   5,   5,   5,   5,   5,   5,   4,
  4,   4,   4,   4,   4,   4,   4,   4,   4,   4,
  4,   4,   4,   4,   4,   3,   3,   3,   3,   3,
  3,   3,   3,   3,   3,   3,   3,   3,   3,   3,
  3,   3,   3,   3,   3,   3,   2,   2,   2,   2,
  2,   2,   2,   2,   2,   2,   2,   2,   2,   2,
  2,   2,   2,   2,   2,   2,   2,   2,   2,   2,
  2,   2,   2,   2,   2,   2,   2,   2,   2,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  0
};

/*----------------------------------------------------------------------------*/

const Ord1 scramblingSeq[NRB_INFO_CRC] =
{
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
  1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0,
  1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1,
  0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1,
  1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1,
  0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0,
  1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0,
  0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0,
  1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0,
  1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0,
  1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1,
  0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1,
  0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1,
  1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0,
  1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1,
  1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0,
  0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0,
  0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1,
  0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1,
  1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1,
  1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0,
  0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1,
  1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0,
  1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0,
  0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1,
  1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1,
  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0,
  0, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1,
  0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1,
  1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 1,
  0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0,
  1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0,
  1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1, 0, 1, 0,
  0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1,
  0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0,
  0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1,
  1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0,
  1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
  1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1,
  1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1,
  0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1,
  0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1,
  1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1,
  1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0,
  0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1,
  0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0,
  0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1,
  0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1,
  1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1,
  1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1,
  1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1,
  0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0,
  1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1,
  1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1,
  1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
  0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0,
  0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1,
  1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0,
  0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1,
  0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0,
  0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1,
  1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0,
  0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1,
  1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1,
  1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0,
  1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1,
  1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1,
  1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1,
  1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,
  1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1,
  0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0,
  0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0,
  0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1,
  0, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1,
  1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1,
  1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0,
  0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0,
  0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1
};

/*----------------------------------------------------------------------------*/

const Ord16 interleaverSeq[NRB_INFO_CRC] =
{
  1102, 522,  812,  232,  0,    116,  290,
  406,  696,  1044, 580,  464,  754,  986,
  174,  58,   928,  348,  870,  638,  1103,
  531,  853,  281,  32,   129,  336,  460,
  734,  1077, 597,  515,  777,  987,  181,
  97,   977,  370,  925,  648,  1105, 562,
  864,  253,  26,   134,  315,  421,  741,
  1078, 608,  512,  798,  989,  178,  64,
  949,  404,  878,  640,  1109, 577,  854,
  269,  5,    145,  331,  459,  719,  1053,
  629,  474,  771,  993,  213,  101,  965,
  360,  901,  670,  1117, 550,  847,  243,
  20,   122,  316,  425,  746,  1088, 594,
  504,  772,  1001, 198,  106,  939,  351,
  891,  646,  1133, 575,  848,  241,  43,
  154,  319,  443,  737,  1098, 613,  471,
  796,  1017, 196,  70,   937,  380,  921,
  677,  1106, 530,  831,  259,  35,   130,
  342,  430,  740,  1084, 601,  466,  782,
  990,  180,  105,  955,  398,  890,  664,
  1111, 552,  825,  274,  7,    148,  302,
  423,  739,  1080, 621,  501,  800,  995,
  229,  89,   970,  399,  924,  639,  1121,
  536,  868,  257,  27,   164,  310,  451,
  700,  1062, 627,  492,  760,  1005, 208,
  98,   953,  363,  881,  659,  1141, 553,
  845,  233,  38,   152,  332,  457,  713,
  1099, 617,  496,  803,  1025, 217,  104,
  929,  361,  892,  643,  1122, 546,  823,
  272,  47,   161,  304,  433,  748,  1059,
  614,  468,  773,  1006, 230,  108,  968,
  374,  918,  644,  1143, 535,  843,  275,
  49,   169,  345,  411,  697,  1056, 619,
  487,  761,  1027, 216,  91,   971,  378,
  899,  655,  1126, 543,  857,  248,  56,
  163,  325,  440,  714,  1072, 591,  472,
  768,  1010, 222,  60,   944,  352,  897,
  658,  564,  855,  255,  51,   138,  329,
  442,  728,  1085, 618,  518,  759,  1035,
  211,  59,   951,  403,  903,  691,  1142,
  538,  830,  251,  4,    142,  340,  434,
  743,  1055, 632,  491,  779,  1026, 182,
  78,   947,  396,  885,  641,  1124, 573,
  842,  287,  46,   139,  326,  407,  738,
  1097, 589,  503,  787,  1008, 186,  71,
  983,  353,  880,  681,  1147, 569,  815,
  258,  16,   156,  317,  456,  720,  1050,
  582,  478,  802,  1031, 218,  86,   954,
  367,  895,  649,  1134, 529,  861,  283,
  29,   158,  307,  437,  726,  1045, 633,
  476,  808,  1018, 179,  96,   979,  394,
  909,  651,  1108, 542,  846,  235,  45,
  127,  309,  454,  724,  1052, 607,  490,
  775,  992,  221,  83,   931,  366,  926,
  673,  1115, 554,  865,  254,  42,   165,
  344,  445,  705,  1054, 611,  510,  809,
  999,  203,  94,   950,  371,  875,  679,
  1129, 556,  837,  260,  2,    166,  337,
  422,  731,  1063, 624,  488,  799,  1013,
  177,  62,   956,  368,  910,  686,  576,
  841,  265,  39,   121,  303,  455,  742,
  1074, 622,  465,  795,  1041, 205,  80,
  961,  358,  923,  645,  540,  832,  279,
  21,   140,  298,  441,  699,  1094, 586,
  508,  758,  1038, 193,  92,   975,  364,
  884,  666,  534,  867,  271,  17,   170,
  299,  438,  733,  1066, 587,  502,  755,
  1032, 215,  100,  967,  384,  883,  661,
  1136, 533,  862,  284,  3,    118,  346,
  450,  702,  1058, 605,  485,  801,  1020,
  214,  66,   980,  372,  886,  665,  1112,
  523,  829,  285,  13,   157,  313,  461,
  732,  1081, 634,  486,  784,  996,  206,
  63,   981,  391,  877,  650,  1123, 541,
  859,  276,  48,   172,  296,  417,  722,
  1096, 625,  479,  789,  1007, 201,  61,
  972,  356,  904,  662,  1145, 544,  821,
  239,  23,   146,  323,  416,  745,  1075,
  581,  469,  791,  1029, 220,  99,   935,
  377,  882,  676,  1130, 574,  818,  277,
  24,   136,  294,  420,  698,  1069, 615,
  480,  780,  1014, 195,  85,   973,  388,
  889,  653,  579,  869,  289,  57,   173,
  347,  463,  753,  1101, 637,  521,  811,
  1043, 231,  115,  985,  405,  927,  695,
  570,  828,  240,  25,   160,  301,  409,
  715,  1068, 620,  470,  788,  1042, 224,
  76,   936,  383,  872,  685,  539,  817,
  268,  31,   155,  322,  448,  708,  1067,
  609,  473,  767,  1040, 227,  109,  964,
  349,  919,  693,  524,  827,  252,  52,
  144,  306,  410,  730,  1092, 588,  511,
  794,  1036, 192,  72,   948,  393,  896,
  663,  1144, 551,  834,  278,  37,   167,
  321,  444,  703,  1057, 623,  481,  793,
  1028, 207,  67,   974,  402,  906,  687,
  1128, 526,  833,  280,  14,   135,  318,
  426,  712,  1047, 604,  514,  769,  1012,
  209,  103,  976,  373,  876,  656,  571,
  850,  262,  22,   159,  295,  439,  709,
  1061, 616,  519,  783,  1039, 225,  68,
  958,  355,  907,  669,  549,  856,  247,
  50,   141,  335,  446,  710,  1065, 596,
  484,  765,  1034, 176,  84,   943,  354,
  873,  694,  1140, 565,  813,  264,  30,
  125,  327,  418,  749,  1083, 590,  493,
  805,  1024, 197,  75,   960,  390,  916,
  674,  1120, 548,  836,  288,  19,   137,
  305,  412,  736,  1046, 600,  489,  762,
  1004, 188,  69,   984,  392,  905,  690,
  1139, 555,  858,  249,  10,   128,  333,
  436,  701,  1086, 603,  517,  792,  1023,
  175,  65,   945,  379,  879,  689,  1118,
  566,  838,  246,  8,    120,  292,  458,
  752,  1089, 598,  498,  804,  1002, 189,
  82,   942,  375,  898,  678,  1135, 558,
  824,  273,  1,    126,  312,  429,  735,
  1073, 626,  513,  797,  1019, 183,  113,
  969,  401,  900,  675,  1110, 537,  826,
  266,  6,    151,  308,  427,  721,  1060,
  599,  467,  806,  994,  194,  114,  962,
  350,  894,  642,  1119, 563,  851,  270,
  53,   147,  297,  435,  706,  1090, 585,
  494,  786,  1003, 223,  95,   966,  357,
  912,  692,  1137, 528,  839,  234,  11,
  150,  311,  462,  711,  1048, 628,  482,
  778,  1021, 219,  102,  930,  400,  917,
  652,  1114, 532,  866,  263,  41,   133,
  320,  413,  729,  1095, 635,  507,  763,
  998,  187,  87,   959,  386,  902,  684,
  1127, 572,  820,  238,  28,   131,  330,
  432,  723,  1100, 584,  509,  757,  1011,
  226,  77,   934,  359,  888,  682,  559,
  835,  286,  12,   162,  328,  415,  725,
  1093, 610,  495,  790,  1037, 184,  90,
  982,  387,  871,  660,  1146, 547,  816,
  267,  15,   124,  293,  424,  744,  1091,
  606,  475,  756,  1030, 202,  79,   963,
  382,  922,  654,  1132, 545,  844,  261,
  55,   123,  300,  447,  718,  1082, 593,
  497,  766,  1016, 228,  111,  957,  385,
  887,  647,  1104, 525,  840,  256,  18,
  168,  334,  414,  707,  1071, 595,  520,
  770,  988,  200,  93,   952,  395,  874,
  688,  1107, 561,  849,  242,  36,   149,
  339,  428,  750,  1051, 631,  477,  807,
  991,  212,  81,   938,  389,  913,  667,
  1113, 567,  814,  250,  40,   119,  338,
  431,  716,  1079, 630,  483,  810,  997,
  190,  73,   946,  369,  914,  672,  1125,
  568,  819,  237,  54,   171,  291,  419,
  747,  1087, 612,  500,  764,  1009, 191,
  107,  933,  381,  911,  668,  578,  852,
  236,  44,   132,  324,  408,  717,  1064,
  583,  499,  781,  1033, 199,  110,  932,
  362,  920,  683,  1138, 560,  822,  245,
  9,    117,  341,  452,  727,  1049, 592,
  506,  776,  1022, 204,  112,  941,  397,
  893,  671,  1116, 557,  860,  282,  34,
  143,  314,  453,  704,  1070, 636,  516,
  774,  1000, 185,  74,   978,  376,  915,
  657,  1131, 527,  863,  244,  33,   153,
  343,  449,  751,  1076, 602,  505,  785,
  1015, 210,  88,   940,  365,  908,  680
};

/*----------------------------------------------------------------------------*/

const Ord16 redVerIndex[8][NRB_CODE_ARQ] =
{
  {
  2305, 2329, 2354, 2379, 2404, 2429, 2454, 2479, 2505, 2530, 2555, 2580,
  2604, 2629, 2654, 2679, 2705, 2730, 2755, 2780, 2805, 2830, 2855, 2881,
  2905, 2930, 2955, 2980, 3005, 3030, 3055, 3081, 3106, 3131, 3156, 3180,
  3205, 3230, 3255, 3281, 3306, 3331, 3356, 3381, 3406, 3431, 2321, 2346,
  2371, 2396, 2421, 2446, 2471, 2497, 2521, 2546, 2571, 2596, 2621, 2646,
  2671, 2697, 2722, 2747, 2772, 2796, 2821, 2846, 2871, 2897, 2922, 2947,
  2972, 2997, 3022, 3047, 3073, 3097, 3122, 3147, 3172, 3197, 3222, 3247,
  3273, 3298, 3323, 3348, 3372, 3397, 3422, 3447, 2313, 2338, 2363, 2388,
  2412, 2437, 2462, 2487, 2513, 2538, 2563, 2588, 2613, 2638, 2663, 2689,
  2713, 2738, 2763, 2788, 2813, 2838, 2863, 2889, 2914, 2939, 2964, 2988,
  3013, 3038, 3063, 3089, 3114, 3139, 3164, 3189, 3214, 3239, 3265, 3289,
  3314, 3339, 3364, 3389, 3414, 3439, 1158, 1183, 1207, 1232, 1258, 1283,
  1308, 1333, 1358, 1383, 1408, 1433, 1459, 1484, 1508, 1533, 1558, 1583,
  1608, 1634, 1659, 1684, 1709, 1734, 1759, 1783, 1808, 1834, 1859, 1884,
  1909, 1934, 1959, 1984, 2009, 2035, 2060, 2084, 2109, 2134, 2159, 2184,
  2210, 2235, 2260, 2285, 1166, 1191, 1216, 1241, 1267, 1292, 1316, 1341,
  1366, 1391, 1416, 1442, 1467, 1492, 1517, 1542, 1567, 1591, 1616, 1642,
  1667, 1692, 1717, 1742, 1767, 1792, 1817, 1843, 1868, 1892, 1917, 1942,
  1967, 1992, 2018, 2043, 2068, 2093, 2118, 2143, 2167, 2192, 2218, 2243,
  2268, 2293, 1174, 1199, 1224, 1250, 1275, 1300, 1325, 1350, 1375, 1399,
  1424, 1450, 1475, 1500, 1525, 1550, 1575, 1600, 1625, 1651, 1676, 1700,
  1725, 1750, 1775, 1800, 1826, 1851, 1876, 1901, 1926, 1951, 1975, 2000,
  2026, 2051, 2076, 2101, 2126, 2151, 2176, 2201, 2227, 2252, 2276, 2301,
  1156, 1181, 1206, 1231, 1257, 1282, 1306, 1331, 1356, 1381, 1406, 1431,
  1457, 1482, 1507, 1532, 1557, 1582, 1606, 1631, 1657, 1682, 1707, 1732,
  1757, 1782, 1807, 1833, 1858, 1882, 1907, 1932, 1957, 1982, 2007, 2033,
  2058, 2083, 2108, 2133, 2158, 2182, 2207, 2233, 2258, 2283, 1164, 1189,
  1214, 1239, 1265, 1290, 1315, 1340, 1365, 1390, 1414, 1439, 1465, 1490,
  1515, 1540, 1565, 1590, 1615, 1641, 1666, 1690, 1715, 1740, 1765, 1790,
  1815, 1841, 1866, 1891, 1916, 1941, 1966, 1990, 2015, 2041, 2066, 2091,
  2116, 2141, 2166, 2191, 2217, 2242, 2266, 2291, 1173, 1198, 1222, 1247,
  1273, 1298, 1323, 1348, 1373, 1398, 1423, 1449, 1474, 1498, 1523, 1548,
  1573, 1598, 1623, 1649, 1674, 1699, 1724, 1749, 1774, 1798, 1823, 1849,
  1874, 1899, 1924, 1949, 1974, 1999, 2025, 2050, 2074, 2099, 2124, 2149,
  2174, 2199, 2225, 2250, 2275, 2300, 2311, 2336, 2361, 2386, 2411, 2436,
  2461, 2486, 2511, 2536, 2561, 2586, 2611, 2636, 2661, 2686, 2712, 2737,
  2762, 2786, 2811, 2836, 2861, 2887, 2912, 2937, 2962, 2987, 3012, 3037,
  3062, 3087, 3112, 3137, 3162, 3187, 3212, 3237, 3262, 3288, 3313, 3338,
  3362, 3387, 3412, 3437, 2319, 2344, 2369, 2394, 2419, 2444, 2469, 2494,
  2520, 2545, 2570, 2594, 2619, 2644, 2669, 2695, 2720, 2745, 2770, 2795,
  2820, 2845, 2870, 2895, 2920, 2945, 2970, 2995, 3020, 3045, 3070, 3096,
  3121, 3146, 3170, 3195, 3220, 3245, 3271, 3296, 3321, 3346, 3371, 3396,
  3421, 3446, 2328, 2353, 2378, 2402, 2427, 2452, 2477, 2503, 2528, 2553,
  2578, 2603, 2628, 2653, 2678, 2703, 2728, 2753, 2778, 2803, 2828, 2853,
  2878, 2904, 2929, 2954, 2978, 3003, 3028, 3053, 3079, 3104, 3129, 3154,
  3179, 3204, 3229, 3254, 3279, 3304, 3329, 3354, 3379, 3404, 3429, 2304,
  1154, 1179, 1204, 1229, 1255, 1280, 1305, 1330, 1355, 1380, 1404, 1429,
  1455, 1480, 1505, 1530, 1555, 1580, 1605, 1630, 1656, 1681, 1705, 1730,
  1755, 1780, 1805, 1831, 1856, 1881, 1906, 1931, 1956, 1980, 2005, 2031,
  2056, 2081, 2106, 2131, 2156, 2181, 2206, 2232, 2257, 2281, 1163, 1188,
  1212, 1237, 1263, 1288, 1313, 1338, 1363, 1388, 1413, 1438, 1464, 1489,
  1513, 1538, 1563, 1588, 1613, 1639, 1664, 1689, 1714, 1739, 1764, 1788,
  1813, 1839, 1864, 1889, 1914, 1939, 1964, 1989, 2014, 2040, 2065, 2089,
  2114, 2139, 2164, 2189, 2215, 2240, 2265, 2290, 1171, 1196, 1221, 1246,
  1272, 1297, 1321, 1346, 1371, 1396, 1421, 1447, 1472, 1497, 1522, 1547,
  1572, 1596, 1621, 1647, 1672, 1697, 1722, 1747, 1772, 1797, 1822, 1848,
  1873, 1897, 1922, 1947, 1972, 1997, 2023, 2048, 2073, 2098, 2123, 2148,
  2172, 2197, 2223, 2248, 2273, 2298, 2309, 2334, 2359, 2384, 2409, 2434,
  2459, 2484, 2510, 2535, 2560, 2585, 2609, 2634, 2659, 2684, 2710, 2735,
  2760, 2785, 2810, 2835, 2860, 2885, 2910, 2935, 2960, 2985, 3010, 3035,
  3060, 3086, 3111, 3136, 3161, 3185, 3210, 3235, 3260, 3286, 3311, 3336,
  3361, 3386, 3411, 3436, 2318, 2343, 2368, 2393, 2417, 2442, 2467, 2492,
  2518, 2543, 2568, 2593, 2618, 2643, 2668, 2693, 2718, 2743, 2768, 2793,
  2818, 2843, 2868, 2894, 2919, 2944, 2969, 2993, 3018, 3043, 3068, 3094,
  3119, 3144, 3169, 3194, 3219, 3244, 3269, 3294, 3319, 3344, 3369, 3394,
  3419, 3444, 2326, 2351, 2376, 2401, 2426, 2451, 2476, 2501, 2526, 2551,
  2576, 2601, 2626, 2651, 2676, 2702, 2727, 2752, 2777, 2801, 2826, 2851,
  2876, 2902, 2927, 2952, 2977, 3002, 3027, 3052, 3077, 3102, 3127, 3152,
  3177, 3202, 3227, 3252, 3278, 3303, 3328, 3353, 3377, 3402, 3427, 3452,
  2308, 2333, 2358, 2383, 2407, 2432, 2457, 2482, 2508, 2533, 2558, 2583,
  2608, 2633, 2658, 2683, 2708, 2733, 2758, 2783, 2808, 2833, 2858, 2884,
  2909, 2934, 2959, 2983, 3008, 3033, 3058, 3084, 3109, 3134, 3159, 3184,
  3209, 3234, 3259, 3284, 3309, 3334, 3359, 3384, 3409, 3434, 2316, 2341,
  2366, 2391, 2416, 2441, 2466, 2491, 2516, 2541, 2566, 2591, 2616, 2641,
  2666, 2692, 2717, 2742, 2767, 2791, 2816, 2841, 2866, 2892, 2917, 2942,
  2967, 2992, 3017, 3042, 3067, 3092, 3117, 3142, 3167, 3192, 3217, 3242,
  3268, 3293, 3318, 3343, 3367, 3392, 3417, 3442, 2324, 2349, 2374, 2399,
  2424, 2449, 2474, 2500, 2525, 2550, 2575, 2599, 2624, 2649, 2674, 2700,
  2725, 2750, 2775, 2800, 2825, 2850, 2875, 2900, 2925, 2950, 2975, 3000,
  3025, 3050, 3076, 3101, 3126, 3151, 3175, 3200, 3225, 3250, 3276, 3301,
  3326, 3351, 3376, 3401, 3426, 3451, 1169, 1194, 1219, 1244, 1270, 1295,
  1320, 1345, 1370, 1394, 1419, 1445, 1470, 1495, 1520, 1545, 1570, 1595,
  1620, 1646, 1671, 1695, 1720, 1745, 1770, 1795, 1820, 1846, 1871, 1896,
  1921, 1946, 1970, 1995, 2021, 2046, 2071, 2096, 2121, 2146, 2171, 2196,
  2222, 2247, 2271, 2296, 1161, 1186, 1211, 1236, 1262, 1287, 1311, 1336,
  1361, 1386, 1411, 1436, 1462, 1487, 1512, 1537, 1562, 1586, 1611, 1637,
  1662, 1687, 1712, 1737, 1762, 1787, 1812, 1838, 1863, 1887, 1912, 1937,
  1962, 1987, 2012, 2038, 2063, 2088, 2113, 2138, 2162, 2187, 2213, 2238,
  2263, 2288, 1178, 1202, 1227, 1253, 1278, 1303, 1328, 1353, 1378, 1403,
  1428, 1454, 1479, 1503, 1528, 1553, 1578, 1603, 1628, 1654, 1679, 1704,
  1729, 1754, 1778, 1803, 1829, 1854, 1879, 1904, 1929, 1954, 1979, 2004,
  2030, 2055, 2079, 2104, 2129, 2154, 2179, 2204, 2230, 2255, 2280, 1153,
  2314, 2339, 2364, 2389, 2414, 2439, 2464, 2489, 2515, 2540, 2565, 2590,
  2614, 2639, 2664, 2690, 2715, 2740, 2765, 2790, 2815, 2840, 2865, 2890,
  2915, 2940, 2965, 2990, 3015, 3040, 3065, 3091, 3116, 3141, 3166, 3190,
  3215, 3240, 3266, 3291, 3316, 3341, 3366, 3391, 3416, 3441, 2306, 2331,
  2356, 2381, 2406, 2431, 2456, 2481, 2506, 2531, 2556, 2581, 2606, 2631,
  2656, 2681, 2707, 2732, 2757, 2782, 2806, 2831, 2856, 2882, 2907, 2932,
  2957, 2982, 3007, 3032, 3057, 3082, 3107, 3132, 3157, 3182, 3207, 3232,
  3257, 3283, 3308, 3333, 3358, 3382, 3407, 3432, 1159, 1184, 1209, 1234,
  1260, 1285, 1310, 1335, 1360, 1385, 1409, 1434, 1460, 1485, 1510, 1535,
  1560, 1585, 1610, 1636, 1661, 1685, 1710, 1735, 1760, 1785, 1810, 1836,
  1861, 1886, 1911, 1936, 1961, 1985, 2010, 2036, 2061, 2086, 2111, 2136,
  2161, 2186, 2212, 2237, 2261, 2286, 2323, 2348, 2373, 2398, 2422, 2447,
  2472, 2498, 2523, 2548, 2573, 2598, 2623, 2648, 2673, 2698, 2723, 2748,
  2773, 2798, 2823, 2848, 2873, 2899, 2924, 2949, 2974, 2998, 3023, 3048,
  3074, 3099, 3124, 3149, 3174, 3199, 3224, 3249, 3274, 3299, 3324, 3349,
  3374, 3399, 3424, 3449, 1176, 1201, 1226, 1252, 1277, 1301, 1326, 1351,
  1376, 1401, 1426, 1452, 1477, 1502, 1527, 1552, 1577, 1601, 1626, 1652,
  1677, 1702, 1727, 1752, 1777, 1802, 1828, 1853, 1877, 1902, 1927, 1952,
  1977, 2002, 2028, 2053, 2078, 2103, 2128, 2153, 2177, 2202, 2228, 2253,
  2278, 1150, 1168, 1193, 1217, 1242, 1268, 1293, 1318, 1343, 1368, 1393,
  1418, 1444, 1469, 1493, 1518, 1543, 1568, 1593, 1618, 1644, 1669, 1694,
  1719, 1744, 1769, 1793, 1818, 1844, 1869, 1894, 1919, 1944, 1969, 1994,
  2020, 2045, 2069, 2094, 2119, 2144, 2169, 2194, 2220, 2245, 2270, 2295
  },
  {
  2305, 2330, 2355, 2380, 2405, 2429, 2454, 2479, 2505, 2530, 2555, 2580,
  2605, 2630, 2655, 2680, 2706, 2730, 2755, 2780, 2805, 2830, 2855, 2881,
  2906, 2931, 2956, 2981, 3005, 3030, 3055, 3081, 3106, 3131, 3156, 3181,
  3206, 3231, 3256, 3282, 3306, 3331, 3356, 3381, 3406, 3431, 2322, 2346,
  2371, 2396, 2421, 2446, 2471, 2497, 2522, 2547, 2572, 2597, 2621, 2646,
  2671, 2697, 2722, 2747, 2772, 2797, 2822, 2847, 2872, 2898, 2922, 2947,
  2972, 2997, 3022, 3047, 3073, 3098, 3123, 3148, 3173, 3197, 3222, 3247,
  3273, 3298, 3323, 3348, 3373, 3398, 3423, 3448, 2313, 2338, 2363, 2388,
  2413, 2438, 2463, 2488, 2514, 2538, 2563, 2588, 2613, 2638, 2663, 2689,
  2714, 2739, 2764, 2789, 2813, 2838, 2863, 2889, 2914, 2939, 2964, 2989,
  3014, 3039, 3064, 3090, 3114, 3139, 3164, 3189, 3214, 3239, 3265, 3290,
  3315, 3340, 3365, 3389, 3414, 3439, 1158, 1183, 1208, 1233, 1259, 1284,
  1309, 1333, 1358, 1383, 1408, 1433, 1459, 1484, 1509, 1534, 1559, 1584,
  1609, 1634, 1659, 1684, 1709, 1734, 1759, 1784, 1809, 1835, 1860, 1885,
  1909, 1934, 1959, 1984, 2009, 2035, 2060, 2085, 2110, 2135, 2160, 2185,
  2210, 2235, 2260, 2285, 1166, 1191, 1216, 1241, 1267, 1292, 1317, 1342,
  1367, 1392, 1417, 1442, 1467, 1492, 1517, 1542, 1567, 1592, 1617, 1643,
  1668, 1693, 1717, 1742, 1767, 1792, 1817, 1843, 1868, 1893, 1918, 1943,
  1968, 1993, 2018, 2043, 2068, 2093, 2118, 2143, 2168, 2193, 2219, 2244,
  2269, 2293, 1175, 1200, 1225, 1250, 1275, 1300, 1325, 1350, 1375, 1400,
  1425, 1451, 1476, 1501, 1525, 1550, 1575, 1600, 1625, 1651, 1676, 1701,
  1726, 1751, 1776, 1801, 1826, 1851, 1876, 1901, 1926, 1951, 1976, 2001,
  2027, 2052, 2077, 2101, 2126, 2151, 2176, 2201, 2227, 2252, 2277, 2302,
  1156, 1181, 1206, 1231, 1257, 1282, 1307, 1332, 1357, 1382, 1407, 1431,
  1457, 1482, 1507, 1532, 1557, 1582, 1607, 1632, 1658, 1683, 1707, 1732,
  1757, 1782, 1807, 1833, 1858, 1883, 1908, 1933, 1958, 1983, 2007, 2033,
  2058, 2083, 2108, 2133, 2158, 2183, 2208, 2234, 2259, 2283, 1165, 1190,
  1215, 1239, 1265, 1290, 1315, 1340, 1365, 1390, 1415, 1440, 1466, 1491,
  1515, 1540, 1565, 1590, 1615, 1641, 1666, 1691, 1716, 1741, 1766, 1791,
  1815, 1841, 1866, 1891, 1916, 1941, 1966, 1991, 2016, 2042, 2067, 2091,
  2116, 2141, 2166, 2191, 2217, 2242, 2267, 2292, 1173, 1198, 1223, 1248,
  1274, 1299, 1323, 1348, 1373, 1398, 1423, 1449, 1474, 1499, 1524, 1549,
  1574, 1599, 1623, 1649, 1674, 1699, 1724, 1749, 1774, 1799, 1824, 1850,
  1875, 1899, 1924, 1949, 1974, 1999, 2025, 2050, 2075, 2100, 2125, 2150,
  2175, 2199, 2225, 2250, 2275, 2300, 2312, 2336, 2361, 2386, 2411, 2436,
  2461, 2486, 2512, 2537, 2562, 2587, 2612, 2636, 2661, 2686, 2712, 2737,
  2762, 2787, 2812, 2837, 2862, 2888, 2912, 2937, 2962, 2987, 3012, 3037,
  3062, 3088, 3113, 3138, 3163, 3188, 3212, 3237, 3262, 3288, 3313, 3338,
  3363, 3388, 3413, 3438, 2320, 2345, 2370, 2395, 2420, 2444, 2469, 2494,
  2520, 2545, 2570, 2595, 2620, 2645, 2670, 2696, 2720, 2745, 2770, 2795,
  2820, 2845, 2870, 2896, 2921, 2946, 2971, 2996, 3020, 3045, 3070, 3096,
  3121, 3146, 3171, 3196, 3221, 3246, 3272, 3296, 3321, 3346, 3371, 3396,
  3421, 3446, 2328, 2353, 2378, 2403, 2428, 2453, 2478, 2504, 2528, 2553,
  2578, 2603, 2628, 2653, 2678, 2704, 2729, 2754, 2779, 2804, 2828, 2853,
  2878, 2904, 2929, 2954, 2979, 3004, 3029, 3054, 3080, 3104, 3129, 3154,
  3179, 3204, 3229, 3254, 3280, 3305, 3330, 3355, 3380, 3404, 3429, 2304,
  1155, 1180, 1205, 1229, 1255, 1280, 1305, 1330, 1355, 1380, 1405, 1430,
  1456, 1481, 1506, 1530, 1555, 1580, 1605, 1630, 1656, 1681, 1706, 1731,
  1756, 1781, 1805, 1831, 1856, 1881, 1906, 1931, 1956, 1981, 2006, 2032,
  2057, 2082, 2106, 2131, 2156, 2181, 2206, 2232, 2257, 2282, 1163, 1188,
  1213, 1238, 1264, 1289, 1314, 1338, 1363, 1388, 1413, 1438, 1464, 1489,
  1514, 1539, 1564, 1589, 1613, 1639, 1664, 1689, 1714, 1739, 1764, 1789,
  1814, 1840, 1865, 1890, 1914, 1939, 1964, 1989, 2014, 2040, 2065, 2090,
  2115, 2140, 2165, 2189, 2215, 2240, 2265, 2290, 1171, 1196, 1221, 1246,
  1272, 1297, 1322, 1347, 1372, 1397, 1421, 1447, 1472, 1497, 1522, 1547,
  1572, 1597, 1622, 1648, 1673, 1698, 1722, 1747, 1772, 1797, 1822, 1848,
  1873, 1898, 1923, 1948, 1973, 1997, 2023, 2048, 2073, 2098, 2123, 2148,
  2173, 2198, 2224, 2249, 2274, 2298, 2310, 2335, 2360, 2385, 2410, 2434,
  2459, 2484, 2510, 2535, 2560, 2585, 2610, 2635, 2660, 2685, 2711, 2735,
  2760, 2785, 2810, 2835, 2860, 2886, 2911, 2936, 2961, 2986, 3010, 3035,
  3060, 3086, 3111, 3136, 3161, 3186, 3211, 3236, 3261, 3287, 3311, 3336,
  3361, 3386, 3411, 3436, 2318, 2343, 2368, 2393, 2418, 2443, 2468, 2493,
  2519, 2543, 2568, 2593, 2618, 2643, 2668, 2694, 2719, 2744, 2769, 2794,
  2818, 2843, 2868, 2894, 2919, 2944, 2969, 2994, 3019, 3044, 3069, 3095,
  3119, 3144, 3169, 3194, 3219, 3244, 3270, 3295, 3320, 3345, 3370, 3394,
  3419, 3444, 2327, 2351, 2376, 2401, 2426, 2451, 2476, 2502, 2527, 2552,
  2577, 2602, 2626, 2651, 2676, 2702, 2727, 2752, 2777, 2802, 2827, 2852,
  2877, 2903, 2927, 2952, 2977, 3002, 3027, 3052, 3078, 3103, 3128, 3153,
  3178, 3202, 3227, 3252, 3278, 3303, 3328, 3353, 3378, 3403, 3428, 1149,
  2308, 2333, 2358, 2383, 2408, 2433, 2458, 2483, 2509, 2533, 2558, 2583,
  2608, 2633, 2658, 2683, 2709, 2734, 2759, 2784, 2808, 2833, 2858, 2884,
  2909, 2934, 2959, 2984, 3009, 3034, 3059, 3085, 3109, 3134, 3159, 3184,
  3209, 3234, 3259, 3285, 3310, 3335, 3360, 3384, 3409, 3434, 2317, 2341,
  2366, 2391, 2416, 2441, 2466, 2491, 2517, 2542, 2567, 2592, 2616, 2641,
  2666, 2692, 2717, 2742, 2767, 2792, 2817, 2842, 2867, 2893, 2917, 2942,
  2967, 2992, 3017, 3042, 3067, 3093, 3118, 3143, 3168, 3192, 3217, 3242,
  3268, 3293, 3318, 3343, 3368, 3393, 3418, 3443, 2325, 2350, 2375, 2400,
  2424, 2449, 2474, 2500, 2525, 2550, 2575, 2600, 2625, 2650, 2675, 2701,
  2725, 2750, 2775, 2800, 2825, 2850, 2875, 2901, 2926, 2951, 2976, 3000,
  3025, 3050, 3076, 3101, 3126, 3151, 3176, 3201, 3226, 3251, 3277, 3301,
  3326, 3351, 3376, 3401, 3426, 3451, 1170, 1195, 1220, 1244, 1270, 1295,
  1320, 1345, 1370, 1395, 1420, 1446, 1471, 1496, 1520, 1545, 1570, 1595,
  1620, 1646, 1671, 1696, 1721, 1746, 1771, 1796, 1820, 1846, 1871, 1896,
  1921, 1946, 1971, 1996, 2022, 2047, 2072, 2096, 2121, 2146, 2171, 2196,
  2222, 2247, 2272, 2297, 1161, 1186, 1211, 1236, 1262, 1287, 1312, 1337,
  1362, 1387, 1412, 1436, 1462, 1487, 1512, 1537, 1562, 1587, 1612, 1638,
  1663, 1688, 1712, 1737, 1762, 1787, 1812, 1838, 1863, 1888, 1913, 1938,
  1963, 1988, 2012, 2038, 2063, 2088, 2113, 2138, 2163, 2188, 2214, 2239,
  2264, 2288, 1178, 1203, 1228, 1254, 1279, 1304, 1328, 1353, 1378, 1403,
  1428, 1454, 1479, 1504, 1529, 1554, 1579, 1604, 1628, 1654, 1679, 1704,
  1729, 1754, 1779, 1804, 1830, 1855, 1880, 1904, 1929, 1954, 1979, 2004,
  2030, 2055, 2080, 2105, 2130, 2155, 2180, 2204, 2230, 2255, 2280, 1153,
  2315, 2340, 2365, 2390, 2415, 2439, 2464, 2489, 2515, 2540, 2565, 2590,
  2615, 2640, 2665, 2691, 2715, 2740, 2765, 2790, 2815, 2840, 2865, 2891,
  2916, 2941, 2966, 2991, 3015, 3040, 3065, 3091, 3116, 3141, 3166, 3191,
  3216, 3241, 3267, 3291, 3316, 3341, 3366, 3391, 3416, 3441, 2307, 2331,
  2356, 2381, 2406, 2431, 2456, 2481, 2507, 2532, 2557, 2582, 2607, 2631,
  2656, 2681, 2707, 2732, 2757, 2782, 2807, 2832, 2857, 2883, 2907, 2932,
  2957, 2982, 3007, 3032, 3057, 3083, 3108, 3133, 3158, 3183, 3207, 3232,
  3257, 3283, 3308, 3333, 3358, 3383, 3408, 3433, 1160, 1185, 1210, 1234,
  1260, 1285, 1310, 1335, 1360, 1385, 1410, 1435, 1461, 1486, 1511, 1535,
  1560, 1585, 1610, 1636, 1661, 1686, 1711, 1736, 1761, 1786, 1810, 1836,
  1861, 1886, 1911, 1936, 1961, 1986, 2011, 2037, 2062, 2087, 2111, 2136,
  2161, 2186, 2212, 2237, 2262, 2287, 2323, 2348, 2373, 2398, 2423, 2448,
  2473, 2499, 2523, 2548, 2573, 2598, 2623, 2648, 2673, 2699, 2724, 2749,
  2774, 2799, 2823, 2848, 2873, 2899, 2924, 2949, 2974, 2999, 3024, 3049,
  3075, 3099, 3124, 3149, 3174, 3199, 3224, 3249, 3275, 3300, 3325, 3350,
  3375, 3399, 3424, 3449, 1176, 1201, 1226, 1252, 1277, 1302, 1327, 1352,
  1377, 1402, 1426, 1452, 1477, 1502, 1527, 1552, 1577, 1602, 1627, 1653,
  1678, 1703, 1727, 1752, 1777, 1802, 1828, 1853, 1878, 1903, 1928, 1953,
  1978, 2002, 2028, 2053, 2078, 2103, 2128, 2153, 2178, 2203, 2229, 2254,
  2279, 1150, 1168, 1193, 1218, 1243, 1269, 1294, 1319, 1343, 1368, 1393,
  1418, 1444, 1469, 1494, 1519, 1544, 1569, 1594, 1618, 1644, 1669, 1694,
  1719, 1744, 1769, 1794, 1819, 1845, 1870, 1895, 1919, 1944, 1969, 1994,
  2020, 2045, 2070, 2095, 2120, 2145, 2170, 2194, 2220, 2245, 2270, 2295
  },
  {
  2305, 2330, 2355, 2380, 2405, 2430, 2455, 2480, 2506, 2531, 2555, 2580,
  2605, 2630, 2655, 2680, 2706, 2731, 2756, 2781, 2806, 2831, 2855, 2881,
  2906, 2931, 2956, 2981, 3006, 3031, 3056, 3082, 3107, 3131, 3156, 3181,
  3206, 3231, 3256, 3282, 3307, 3332, 3357, 3382, 3407, 3431, 2322, 2347,
  2372, 2397, 2422, 2447, 2471, 2497, 2522, 2547, 2572, 2597, 2622, 2647,
  2672, 2698, 2723, 2747, 2772, 2797, 2822, 2847, 2872, 2898, 2923, 2948,
  2973, 2998, 3023, 3047, 3073, 3098, 3123, 3148, 3173, 3198, 3223, 3248,
  3274, 3299, 3323, 3348, 3373, 3398, 3423, 3448, 2314, 2339, 2363, 2388,
  2413, 2438, 2463, 2488, 2514, 2539, 2564, 2589, 2614, 2639, 2663, 2689,
  2714, 2739, 2764, 2789, 2814, 2839, 2864, 2890, 2915, 2939, 2964, 2989,
  3014, 3039, 3064, 3090, 3115, 3140, 3165, 3190, 3215, 3239, 3265, 3290,
  3315, 3340, 3365, 3390, 3415, 3440, 1157, 1182, 1207, 1232, 1257, 1282,
  1307, 1332, 1357, 1382, 1407, 1432, 1458, 1483, 1508, 1533, 1557, 1582,
  1607, 1632, 1658, 1683, 1708, 1733, 1758, 1783, 1808, 1833, 1858, 1883,
  1908, 1933, 1958, 1983, 2008, 2034, 2059, 2084, 2109, 2133, 2158, 2183,
  2208, 2234, 2259, 2284, 1165, 1190, 1215, 1240, 1266, 1291, 1316, 1341,
  1365, 1390, 1415, 1440, 1466, 1491, 1516, 1541, 1566, 1591, 1616, 1641,
  1666, 1691, 1716, 1741, 1766, 1791, 1816, 1842, 1867, 1892, 1917, 1941,
  1966, 1991, 2016, 2042, 2067, 2092, 2117, 2142, 2167, 2192, 2217, 2242,
  2267, 2292, 1173, 1198, 1223, 1248, 1274, 1299, 1324, 1349, 1374, 1399,
  1424, 1449, 1474, 1499, 1524, 1549, 1574, 1599, 1624, 1650, 1675, 1700,
  1725, 1749, 1774, 1799, 1824, 1850, 1875, 1900, 1925, 1950, 1975, 2000,
  2025, 2050, 2075, 2100, 2125, 2150, 2175, 2200, 2226, 2251, 2276, 2301,
  1155, 1180, 1205, 1230, 1256, 1281, 1306, 1331, 1355, 1380, 1405, 1430,
  1456, 1481, 1506, 1531, 1556, 1581, 1606, 1631, 1656, 1681, 1706, 1731,
  1756, 1781, 1806, 1832, 1857, 1882, 1907, 1931, 1956, 1981, 2006, 2032,
  2057, 2082, 2107, 2132, 2157, 2182, 2207, 2232, 2257, 2282, 1163, 1188,
  1213, 1238, 1264, 1289, 1314, 1339, 1364, 1389, 1414, 1439, 1464, 1489,
  1514, 1539, 1564, 1589, 1614, 1640, 1665, 1690, 1715, 1739, 1764, 1789,
  1814, 1840, 1865, 1890, 1915, 1940, 1965, 1990, 2015, 2040, 2065, 2090,
  2115, 2140, 2165, 2190, 2216, 2241, 2266, 2291, 1172, 1197, 1222, 1247,
  1272, 1297, 1322, 1347, 1372, 1397, 1422, 1448, 1473, 1498, 1523, 1547,
  1572, 1597, 1622, 1648, 1673, 1698, 1723, 1748, 1773, 1798, 1823, 1848,
  1873, 1898, 1923, 1948, 1973, 1998, 2024, 2049, 2074, 2099, 2123, 2148,
  2173, 2198, 2224, 2249, 2274, 2299, 2312, 2337, 2362, 2387, 2412, 2437,
  2461, 2486, 2512, 2537, 2562, 2587, 2612, 2637, 2662, 2687, 2713, 2737,
  2762, 2787, 2812, 2837, 2862, 2888, 2913, 2938, 2963, 2988, 3013, 3037,
  3062, 3088, 3113, 3138, 3163, 3188, 3213, 3238, 3263, 3289, 3313, 3338,
  3363, 3388, 3413, 3438, 2320, 2345, 2370, 2395, 2420, 2445, 2470, 2495,
  2521, 2545, 2570, 2595, 2620, 2645, 2670, 2696, 2721, 2746, 2771, 2796,
  2821, 2845, 2870, 2896, 2921, 2946, 2971, 2996, 3021, 3046, 3071, 3097,
  3121, 3146, 3171, 3196, 3221, 3246, 3272, 3297, 3322, 3347, 3372, 3397,
  3421, 3446, 2329, 2353, 2378, 2403, 2428, 2453, 2478, 2504, 2529, 2554,
  2579, 2604, 2629, 2653, 2678, 2704, 2729, 2754, 2779, 2804, 2829, 2854,
  2879, 2905, 2929, 2954, 2979, 3004, 3029, 3054, 3080, 3105, 3130, 3155,
  3180, 3205, 3229, 3254, 3280, 3305, 3330, 3355, 3380, 3405, 3430, 1152,
  1154, 1178, 1203, 1228, 1254, 1279, 1304, 1329, 1354, 1379, 1404, 1429,
  1454, 1479, 1504, 1529, 1554, 1579, 1604, 1629, 1655, 1680, 1705, 1730,
  1754, 1779, 1804, 1830, 1855, 1880, 1905, 1930, 1955, 1980, 2005, 2030,
  2055, 2080, 2105, 2130, 2155, 2180, 2205, 2231, 2256, 2281, 1162, 1187,
  1212, 1237, 1262, 1287, 1312, 1337, 1362, 1387, 1412, 1437, 1463, 1488,
  1513, 1538, 1562, 1587, 1612, 1638, 1663, 1688, 1713, 1738, 1763, 1788,
  1813, 1838, 1863, 1888, 1913, 1938, 1963, 1988, 2013, 2039, 2064, 2089,
  2114, 2138, 2163, 2188, 2214, 2239, 2264, 2289, 1170, 1195, 1220, 1245,
  1271, 1296, 1321, 1346, 1370, 1395, 1420, 1446, 1471, 1496, 1521, 1546,
  1571, 1596, 1621, 1646, 1671, 1696, 1721, 1746, 1771, 1796, 1821, 1847,
  1872, 1897, 1922, 1946, 1971, 1996, 2022, 2047, 2072, 2097, 2122, 2147,
  2172, 2197, 2222, 2247, 2272, 2297, 2310, 2335, 2360, 2385, 2410, 2435,
  2460, 2485, 2511, 2536, 2560, 2585, 2610, 2635, 2660, 2685, 2711, 2736,
  2761, 2786, 2811, 2835, 2860, 2886, 2911, 2936, 2961, 2986, 3011, 3036,
  3061, 3087, 3112, 3136, 3161, 3186, 3211, 3236, 3261, 3287, 3312, 3337,
  3362, 3387, 3411, 3436, 2319, 2344, 2368, 2393, 2418, 2443, 2468, 2493,
  2519, 2544, 2569, 2594, 2619, 2643, 2668, 2694, 2719, 2744, 2769, 2794,
  2819, 2844, 2869, 2895, 2920, 2944, 2969, 2994, 3019, 3044, 3069, 3095,
  3120, 3145, 3170, 3195, 3219, 3244, 3270, 3295, 3320, 3345, 3370, 3395,
  3420, 3445, 2327, 2352, 2377, 2402, 2427, 2451, 2476, 2502, 2527, 2552,
  2577, 2602, 2627, 2652, 2677, 2703, 2728, 2752, 2777, 2802, 2827, 2852,
  2877, 2903, 2928, 2953, 2978, 3003, 3027, 3052, 3078, 3103, 3128, 3153,
  3178, 3203, 3228, 3253, 3279, 3304, 3328, 3353, 3378, 3403, 3428, 1149,
  2309, 2334, 2358, 2383, 2408, 2433, 2458, 2483, 2509, 2534, 2559, 2584,
  2609, 2634, 2658, 2683, 2709, 2734, 2759, 2784, 2809, 2834, 2859, 2885,
  2910, 2934, 2959, 2984, 3009, 3034, 3059, 3085, 3110, 3135, 3160, 3185,
  3210, 3234, 3259, 3285, 3310, 3335, 3360, 3385, 3410, 3435, 2317, 2342,
  2367, 2392, 2417, 2442, 2466, 2491, 2517, 2542, 2567, 2592, 2617, 2642,
  2667, 2693, 2718, 2742, 2767, 2792, 2817, 2842, 2867, 2893, 2918, 2943,
  2968, 2993, 3018, 3042, 3067, 3093, 3118, 3143, 3168, 3193, 3218, 3243,
  3269, 3294, 3318, 3343, 3368, 3393, 3418, 3443, 2325, 2350, 2375, 2400,
  2425, 2450, 2475, 2501, 2526, 2550, 2575, 2600, 2625, 2650, 2675, 2701,
  2726, 2751, 2776, 2801, 2826, 2850, 2875, 2901, 2926, 2951, 2976, 3001,
  3026, 3051, 3077, 3102, 3126, 3151, 3176, 3201, 3226, 3251, 3277, 3302,
  3327, 3352, 3377, 3402, 3426, 3451, 1168, 1193, 1218, 1243, 1269, 1294,
  1319, 1344, 1369, 1394, 1419, 1444, 1469, 1494, 1519, 1544, 1569, 1594,
  1619, 1645, 1670, 1695, 1720, 1744, 1769, 1794, 1819, 1845, 1870, 1895,
  1920, 1945, 1970, 1995, 2020, 2045, 2070, 2095, 2120, 2145, 2170, 2195,
  2221, 2246, 2271, 2296, 1160, 1185, 1210, 1235, 1261, 1286, 1311, 1336,
  1360, 1385, 1410, 1435, 1461, 1486, 1511, 1536, 1561, 1586, 1611, 1636,
  1661, 1686, 1711, 1736, 1761, 1786, 1811, 1837, 1862, 1887, 1912, 1936,
  1961, 1986, 2011, 2037, 2062, 2087, 2112, 2137, 2162, 2187, 2212, 2237,
  2262, 2287, 1177, 1202, 1227, 1252, 1277, 1302, 1327, 1352, 1377, 1402,
  1427, 1453, 1478, 1503, 1528, 1552, 1577, 1602, 1627, 1653, 1678, 1703,
  1728, 1753, 1778, 1803, 1828, 1853, 1878, 1903, 1928, 1953, 1978, 2003,
  2029, 2054, 2079, 2104, 2128, 2153, 2178, 2203, 2229, 2254, 2279, 3453,
  2315, 2340, 2365, 2390, 2415, 2440, 2465, 2490, 2516, 2541, 2565, 2590,
  2615, 2640, 2665, 2691, 2716, 2741, 2766, 2791, 2816, 2840, 2865, 2891,
  2916, 2941, 2966, 2991, 3016, 3041, 3066, 3092, 3117, 3141, 3166, 3191,
  3216, 3241, 3267, 3292, 3317, 3342, 3367, 3392, 3416, 3441, 2307, 2332,
  2357, 2382, 2407, 2432, 2456, 2481, 2507, 2532, 2557, 2582, 2607, 2632,
  2657, 2682, 2708, 2733, 2757, 2782, 2807, 2832, 2857, 2883, 2908, 2933,
  2958, 2983, 3008, 3032, 3057, 3083, 3108, 3133, 3158, 3183, 3208, 3233,
  3258, 3284, 3309, 3333, 3358, 3383, 3408, 3433, 1158, 1183, 1208, 1233,
  1259, 1284, 1309, 1334, 1359, 1384, 1409, 1434, 1459, 1484, 1509, 1534,
  1559, 1584, 1609, 1635, 1660, 1685, 1710, 1734, 1759, 1784, 1809, 1835,
  1860, 1885, 1910, 1935, 1960, 1985, 2010, 2035, 2060, 2085, 2110, 2135,
  2160, 2185, 2211, 2236, 2261, 2286, 2324, 2349, 2373, 2398, 2423, 2448,
  2473, 2499, 2524, 2549, 2574, 2599, 2624, 2648, 2673, 2699, 2724, 2749,
  2774, 2799, 2824, 2849, 2874, 2900, 2925, 2949, 2974, 2999, 3024, 3049,
  3075, 3100, 3125, 3150, 3175, 3200, 3224, 3249, 3275, 3300, 3325, 3350,
  3375, 3400, 3425, 3450, 1175, 1200, 1225, 1251, 1276, 1301, 1326, 1350,
  1375, 1400, 1425, 1451, 1476, 1501, 1526, 1551, 1576, 1601, 1626, 1651,
  1676, 1701, 1726, 1751, 1776, 1801, 1827, 1852, 1877, 1902, 1926, 1951,
  1976, 2001, 2027, 2052, 2077, 2102, 2127, 2152, 2177, 2202, 2227, 2252,
  2277, 2302, 1167, 1192, 1217, 1242, 1267, 1292, 1317, 1342, 1367, 1392,
  1417, 1443, 1468, 1493, 1518, 1542, 1567, 1592, 1617, 1643, 1668, 1693,
  1718, 1743, 1768, 1793, 1818, 1843, 1868, 1893, 1918, 1943, 1968, 1993,
  2019, 2044, 2069, 2094, 2118, 2143, 2168, 2193, 2219, 2244, 2269, 2294
  },
  {
  2306, 2331, 2356, 2380, 2405, 2430, 2455, 2480, 2506, 2531, 2556, 2581,
  2606, 2631, 2656, 2680, 2706, 2731, 2756, 2781, 2806, 2831, 2856, 2882,
  2907, 2932, 2956, 2981, 3006, 3031, 3056, 3082, 3107, 3132, 3157, 3182,
  3207, 3232, 3256, 3282, 3307, 3332, 3357, 3382, 3407, 3432, 2322, 2347,
  2372, 2397, 2422, 2447, 2472, 2498, 2523, 2548, 2572, 2597, 2622, 2647,
  2672, 2698, 2723, 2748, 2773, 2798, 2823, 2848, 2872, 2898, 2923, 2948,
  2973, 2998, 3023, 3048, 3074, 3099, 3124, 3148, 3173, 3198, 3223, 3248,
  3274, 3299, 3324, 3349, 3374, 3399, 3424, 3448, 2314, 2339, 2364, 2389,
  2414, 2439, 2464, 2488, 2514, 2539, 2564, 2589, 2614, 2639, 2664, 2690,
  2715, 2740, 2764, 2789, 2814, 2839, 2864, 2890, 2915, 2940, 2965, 2990,
  3015, 3040, 3064, 3090, 3115, 3140, 3165, 3190, 3215, 3240, 3266, 3291,
  3316, 3340, 3365, 3390, 3415, 3440, 1157, 1182, 1207, 1232, 1258, 1283,
  1308, 1333, 1358, 1382, 1407, 1432, 1458, 1483, 1508, 1533, 1558, 1583,
  1608, 1634, 1658, 1683, 1708, 1733, 1758, 1783, 1808, 1834, 1859, 1884,
  1909, 1934, 1958, 1983, 2008, 2034, 2059, 2084, 2109, 2134, 2159, 2184,
  2210, 2234, 2259, 2284, 1166, 1190, 1215, 1240, 1266, 1291, 1316, 1341,
  1366, 1391, 1416, 1442, 1466, 1491, 1516, 1541, 1566, 1591, 1616, 1642,
  1667, 1692, 1717, 1742, 1766, 1791, 1816, 1842, 1867, 1892, 1917, 1942,
  1967, 1992, 2018, 2042, 2067, 2092, 2117, 2142, 2167, 2192, 2218, 2243,
  2268, 2293, 1174, 1199, 1224, 1250, 1274, 1299, 1324, 1349, 1374, 1399,
  1424, 1450, 1475, 1500, 1525, 1550, 1574, 1599, 1624, 1650, 1675, 1700,
  1725, 1750, 1775, 1800, 1826, 1850, 1875, 1900, 1925, 1950, 1975, 2000,
  2026, 2051, 2076, 2101, 2126, 2150, 2175, 2200, 2226, 2251, 2276, 2301,
  1156, 1180, 1205, 1230, 1256, 1281, 1306, 1331, 1356, 1381, 1406, 1431,
  1457, 1481, 1506, 1531, 1556, 1581, 1606, 1631, 1657, 1682, 1707, 1732,
  1756, 1781, 1806, 1832, 1857, 1882, 1907, 1932, 1957, 1982, 2007, 2033,
  2057, 2082, 2107, 2132, 2157, 2182, 2207, 2233, 2258, 2283, 1164, 1189,
  1214, 1239, 1265, 1289, 1314, 1339, 1364, 1389, 1414, 1439, 1465, 1490,
  1515, 1540, 1564, 1589, 1614, 1640, 1665, 1690, 1715, 1740, 1765, 1790,
  1815, 1841, 1865, 1890, 1915, 1940, 1965, 1990, 2015, 2041, 2066, 2091,
  2116, 2140, 2165, 2190, 2216, 2241, 2266, 2291, 1172, 1197, 1222, 1247,
  1273, 1298, 1323, 1348, 1372, 1397, 1422, 1448, 1473, 1498, 1523, 1548,
  1573, 1598, 1623, 1649, 1673, 1698, 1723, 1748, 1773, 1798, 1823, 1849,
  1874, 1899, 1924, 1948, 1973, 1998, 2024, 2049, 2074, 2099, 2124, 2149,
  2174, 2199, 2225, 2249, 2274, 2299, 2312, 2337, 2362, 2387, 2412, 2437,
  2462, 2487, 2513, 2538, 2563, 2587, 2612, 2637, 2662, 2687, 2713, 2738,
  2763, 2788, 2813, 2838, 2862, 2888, 2913, 2938, 2963, 2988, 3013, 3038,
  3063, 3089, 3114, 3139, 3163, 3188, 3213, 3238, 3263, 3289, 3314, 3339,
  3364, 3389, 3414, 3438, 2321, 2346, 2371, 2395, 2420, 2445, 2470, 2495,
  2521, 2546, 2571, 2596, 2621, 2646, 2670, 2696, 2721, 2746, 2771, 2796,
  2821, 2846, 2871, 2897, 2922, 2947, 2971, 2996, 3021, 3046, 3071, 3097,
  3122, 3147, 3172, 3197, 3222, 3246, 3272, 3297, 3322, 3347, 3372, 3397,
  3422, 3447, 2329, 2354, 2379, 2404, 2429, 2454, 2478, 2504, 2529, 2554,
  2579, 2604, 2629, 2654, 2679, 2705, 2730, 2755, 2779, 2804, 2829, 2854,
  2879, 2905, 2930, 2955, 2980, 3005, 3030, 3054, 3080, 3105, 3130, 3155,
  3180, 3205, 3230, 3255, 3281, 3306, 3331, 3355, 3380, 3405, 3430, 1152,
  1154, 1179, 1204, 1229, 1255, 1279, 1304, 1329, 1354, 1379, 1404, 1429,
  1455, 1480, 1505, 1530, 1555, 1579, 1604, 1629, 1655, 1680, 1705, 1730,
  1755, 1780, 1805, 1831, 1855, 1880, 1905, 1930, 1955, 1980, 2005, 2031,
  2056, 2081, 2106, 2131, 2155, 2180, 2205, 2231, 2256, 2281, 1162, 1187,
  1212, 1237, 1263, 1288, 1313, 1338, 1363, 1387, 1412, 1437, 1463, 1488,
  1513, 1538, 1563, 1588, 1613, 1639, 1663, 1688, 1713, 1738, 1763, 1788,
  1813, 1839, 1864, 1889, 1914, 1939, 1963, 1988, 2013, 2039, 2064, 2089,
  2114, 2139, 2164, 2189, 2215, 2239, 2264, 2289, 1171, 1195, 1220, 1245,
  1271, 1296, 1321, 1346, 1371, 1396, 1421, 1447, 1471, 1496, 1521, 1546,
  1571, 1596, 1621, 1647, 1672, 1697, 1722, 1747, 1771, 1796, 1821, 1847,
  1872, 1897, 1922, 1947, 1972, 1997, 2023, 2047, 2072, 2097, 2122, 2147,
  2172, 2197, 2223, 2248, 2273, 2298, 2311, 2336, 2361, 2385, 2410, 2435,
  2460, 2485, 2511, 2536, 2561, 2586, 2611, 2636, 2661, 2685, 2711, 2736,
  2761, 2786, 2811, 2836, 2861, 2887, 2912, 2937, 2961, 2986, 3011, 3036,
  3061, 3087, 3112, 3137, 3162, 3187, 3212, 3237, 3261, 3287, 3312, 3337,
  3362, 3387, 3412, 3437, 2319, 2344, 2369, 2394, 2419, 2444, 2469, 2493,
  2519, 2544, 2569, 2594, 2619, 2644, 2669, 2695, 2720, 2745, 2769, 2794,
  2819, 2844, 2869, 2895, 2920, 2945, 2970, 2995, 3020, 3045, 3069, 3095,
  3120, 3145, 3170, 3195, 3220, 3245, 3271, 3296, 3321, 3345, 3370, 3395,
  3420, 3445, 2327, 2352, 2377, 2402, 2427, 2452, 2477, 2503, 2528, 2553,
  2577, 2602, 2627, 2652, 2677, 2703, 2728, 2753, 2778, 2803, 2828, 2853,
  2877, 2903, 2928, 2953, 2978, 3003, 3028, 3053, 3079, 3104, 3129, 3153,
  3178, 3203, 3228, 3253, 3279, 3304, 3329, 3354, 3379, 3404, 3429, 1149,
  2309, 2334, 2359, 2384, 2409, 2434, 2459, 2483, 2509, 2534, 2559, 2584,
  2609, 2634, 2659, 2684, 2710, 2735, 2760, 2784, 2809, 2834, 2859, 2885,
  2910, 2935, 2960, 2985, 3010, 3035, 3059, 3085, 3110, 3135, 3160, 3185,
  3210, 3235, 3260, 3286, 3311, 3336, 3360, 3385, 3410, 3435, 2317, 2342,
  2367, 2392, 2417, 2442, 2467, 2492, 2518, 2543, 2568, 2592, 2617, 2642,
  2667, 2693, 2718, 2743, 2768, 2793, 2818, 2843, 2867, 2893, 2918, 2943,
  2968, 2993, 3018, 3043, 3068, 3094, 3119, 3144, 3168, 3193, 3218, 3243,
  3269, 3294, 3319, 3344, 3369, 3394, 3419, 3443, 2326, 2351, 2376, 2400,
  2425, 2450, 2475, 2501, 2526, 2551, 2576, 2601, 2626, 2651, 2675, 2701,
  2726, 2751, 2776, 2801, 2826, 2851, 2876, 2902, 2927, 2952, 2976, 3001,
  3026, 3051, 3077, 3102, 3127, 3152, 3177, 3202, 3227, 3251, 3277, 3302,
  3327, 3352, 3377, 3402, 3427, 3452, 1169, 1194, 1219, 1244, 1270, 1294,
  1319, 1344, 1369, 1394, 1419, 1445, 1470, 1495, 1520, 1545, 1569, 1594,
  1619, 1645, 1670, 1695, 1720, 1745, 1770, 1795, 1820, 1846, 1870, 1895,
  1920, 1945, 1970, 1995, 2021, 2046, 2071, 2096, 2121, 2145, 2170, 2195,
  2221, 2246, 2271, 2296, 1161, 1185, 1210, 1235, 1261, 1286, 1311, 1336,
  1361, 1386, 1411, 1436, 1462, 1486, 1511, 1536, 1561, 1586, 1611, 1637,
  1662, 1687, 1712, 1737, 1761, 1786, 1811, 1837, 1862, 1887, 1912, 1937,
  1962, 1987, 2012, 2038, 2062, 2087, 2112, 2137, 2162, 2187, 2213, 2238,
  2263, 2288, 1177, 1202, 1227, 1253, 1278, 1303, 1328, 1353, 1377, 1402,
  1427, 1453, 1478, 1503, 1528, 1553, 1578, 1603, 1628, 1654, 1678, 1703,
  1728, 1753, 1778, 1803, 1829, 1854, 1879, 1904, 1929, 1953, 1978, 2003,
  2029, 2054, 2079, 2104, 2129, 2154, 2179, 2204, 2230, 2254, 2279, 3453,
  2316, 2341, 2366, 2390, 2415, 2440, 2465, 2490, 2516, 2541, 2566, 2591,
  2616, 2641, 2665, 2691, 2716, 2741, 2766, 2791, 2816, 2841, 2866, 2892,
  2917, 2942, 2966, 2991, 3016, 3041, 3066, 3092, 3117, 3142, 3167, 3192,
  3217, 3241, 3267, 3292, 3317, 3342, 3367, 3392, 3417, 3442, 2307, 2332,
  2357, 2382, 2407, 2432, 2457, 2482, 2508, 2533, 2558, 2582, 2607, 2632,
  2657, 2682, 2708, 2733, 2758, 2783, 2808, 2833, 2857, 2883, 2908, 2933,
  2958, 2983, 3008, 3033, 3058, 3084, 3109, 3134, 3158, 3183, 3208, 3233,
  3258, 3284, 3309, 3334, 3359, 3384, 3409, 3433, 1159, 1184, 1209, 1234,
  1260, 1284, 1309, 1334, 1359, 1384, 1409, 1434, 1460, 1485, 1510, 1535,
  1560, 1584, 1609, 1635, 1660, 1685, 1710, 1735, 1760, 1785, 1810, 1836,
  1860, 1885, 1910, 1935, 1960, 1985, 2010, 2036, 2061, 2086, 2111, 2136,
  2160, 2185, 2211, 2236, 2261, 2286, 2324, 2349, 2374, 2399, 2424, 2449,
  2473, 2499, 2524, 2549, 2574, 2599, 2624, 2649, 2674, 2700, 2725, 2750,
  2774, 2799, 2824, 2849, 2874, 2900, 2925, 2950, 2975, 3000, 3025, 3049,
  3075, 3100, 3125, 3150, 3175, 3200, 3225, 3250, 3276, 3301, 3326, 3350,
  3375, 3400, 3425, 3450, 1176, 1200, 1225, 1251, 1276, 1301, 1326, 1351,
  1376, 1401, 1426, 1452, 1476, 1501, 1526, 1551, 1576, 1601, 1626, 1652,
  1677, 1702, 1727, 1752, 1776, 1801, 1827, 1852, 1877, 1902, 1927, 1952,
  1977, 2002, 2028, 2052, 2077, 2102, 2127, 2152, 2177, 2202, 2228, 2253,
  2278, 1150, 1167, 1192, 1217, 1242, 1268, 1293, 1318, 1343, 1368, 1392,
  1417, 1443, 1468, 1493, 1518, 1543, 1568, 1593, 1618, 1644, 1668, 1693,
  1718, 1743, 1768, 1793, 1818, 1844, 1869, 1894, 1919, 1944, 1968, 1993,
  2019, 2044, 2069, 2094, 2119, 2144, 2169, 2194, 2220, 2244, 2269, 2294
  },
  {
  0,    30,   60,   90,   120,  150,  180,  210,  240,  270,  300,  330,
  360,  390,  420,  450,  480,  510,  540,  570,  600,  630,  660,  690,
  720,  750,  780,  810,  840,  870,  900,  927,  942,  957,  972,  987,
  1002, 1017, 1032, 1047, 1062, 1077, 1092, 1107, 1122, 1137, 20,   50,
  80,   110,  140,  170,  200,  230,  260,  290,  320,  350,  380,  410,
  440,  470,  500,  530,  560,  590,  620,  650,  680,  710,  740,  770,
  800,  830,  860,  890,  920,  937,  952,  967,  982,  997,  1012, 1027,
  1042, 1057, 1072, 1087, 1102, 1117, 1132, 1147, 10,   40,   70,   100,
  130,  160,  190,  220,  250,  280,  310,  340,  370,  400,  430,  460,
  490,  520,  550,  580,  610,  640,  670,  700,  730,  760,  790,  820,
  850,  880,  910,  932,  947,  962,  977,  992,  1007, 1022, 1037, 1052,
  1067, 1082, 1097, 1112, 1127, 1142, 5,    35,   65,   95,   125,  155,
  185,  215,  245,  275,  305,  335,  365,  395,  425,  455,  485,  515,
  545,  575,  605,  635,  665,  695,  725,  755,  785,  815,  845,  875,
  905,  1179, 2405, 1330, 2557, 1482, 2709, 1634, 2859, 1785, 3011, 1937,
  3163, 2088, 3315, 2240, 15,   45,   75,   105,  135,  165,  195,  225,
  255,  285,  315,  345,  375,  405,  435,  465,  495,  525,  555,  585,
  615,  645,  675,  705,  735,  765,  795,  825,  855,  885,  915,  2355,
  1280, 2507, 1431, 2657, 1583, 2809, 1735, 2961, 1886, 3113, 2038, 3265,
  2189, 3415, 25,   55,   85,   115,  145,  175,  205,  235,  265,  295,
  325,  355,  385,  415,  445,  475,  505,  535,  565,  595,  625,  655,
  685,  715,  745,  775,  805,  835,  865,  895,  2305, 1229, 2455, 1381,
  2607, 1532, 2759, 1684, 2911, 1836, 3061, 1987, 3213, 2139, 3365, 2290,
  3,    33,   63,   93,   123,  153,  183,  213,  243,  273,  303,  333,
  363,  393,  423,  453,  483,  513,  543,  573,  603,  633,  663,  693,
  723,  753,  783,  813,  843,  873,  903,  2325, 1250, 2475, 1401, 2627,
  1553, 2779, 1704, 2931, 1856, 3083, 2007, 3233, 2159, 3385, 13,   43,
  73,   103,  133,  163,  193,  223,  253,  283,  313,  343,  373,  403,
  433,  463,  493,  523,  553,  583,  613,  643,  673,  703,  733,  763,
  793,  823,  853,  883,  913,  1199, 2425, 1351, 2577, 1502, 2729, 1654,
  2881, 1805, 3031, 1957, 3183, 2108, 3335, 2260, 23,   53,   83,   113,
  143,  173,  203,  233,  263,  293,  323,  353,  383,  413,  443,  473,
  503,  533,  563,  593,  623,  653,  683,  713,  743,  773,  803,  833,
  863,  893,  923,  2375, 1300, 2527, 1452, 2677, 1603, 2829, 1755, 2981,
  1906, 3133, 2058, 3285, 2210, 3435, 8,    38,   68,   98,   128,  158,
  188,  218,  248,  278,  308,  338,  368,  398,  428,  458,  488,  518,
  548,  578,  608,  638,  668,  698,  728,  758,  788,  818,  848,  878,
  908,  931,  946,  961,  976,  991,  1006, 1021, 1036, 1051, 1066, 1081,
  1096, 1111, 1126, 1141, 18,   48,   78,   108,  138,  168,  198,  228,
  258,  288,  318,  348,  378,  408,  438,  468,  498,  528,  558,  588,
  618,  648,  678,  708,  738,  768,  798,  828,  858,  888,  918,  936,
  951,  966,  981,  996,  1011, 1026, 1041, 1056, 1071, 1086, 1101, 1116,
  1131, 1146, 28,   58,   88,   118,  148,  178,  208,  238,  268,  298,
  328,  358,  388,  418,  448,  478,  508,  538,  568,  598,  628,  658,
  688,  718,  748,  778,  808,  838,  868,  898,  926,  941,  956,  971,
  986,  1001, 1016, 1031, 1046, 1061, 1076, 1091, 1106, 1121, 1136, 3454,
  1,    31,   61,   91,   121,  151,  181,  211,  241,  271,  301,  331,
  361,  391,  421,  451,  481,  511,  541,  571,  601,  631,  661,  691,
  721,  751,  781,  811,  841,  871,  901,  1169, 2395, 1320, 2547, 1472,
  2699, 1623, 2849, 1775, 3001, 1927, 3153, 2078, 3305, 2230, 11,   41,
  71,   101,  131,  161,  191,  221,  251,  281,  311,  341,  371,  401,
  431,  461,  491,  521,  551,  581,  611,  641,  671,  701,  731,  761,
  791,  821,  851,  881,  911,  2345, 1270, 2497, 1421, 2647, 1573, 2799,
  1724, 2951, 1876, 3103, 2028, 3253, 2179, 3405, 21,   51,   81,   111,
  141,  171,  201,  231,  261,  291,  321,  351,  381,  411,  441,  471,
  501,  531,  561,  591,  621,  651,  681,  711,  741,  771,  801,  831,
  861,  891,  921,  1219, 2445, 1371, 2597, 1522, 2749, 1674, 2901, 1826,
  3051, 1977, 3203, 2129, 3355, 2280, 6,    36,   66,   96,   126,  156,
  186,  216,  246,  276,  306,  336,  366,  396,  426,  456,  486,  516,
  546,  576,  606,  636,  666,  696,  726,  756,  786,  816,  846,  876,
  906,  930,  945,  960,  975,  990,  1005, 1020, 1035, 1050, 1065, 1080,
  1095, 1110, 1125, 1140, 16,   46,   76,   106,  136,  166,  196,  226,
  256,  286,  316,  346,  376,  406,  436,  466,  496,  526,  556,  586,
  616,  646,  676,  706,  736,  766,  796,  826,  856,  886,  916,  935,
  950,  965,  980,  995,  1010, 1025, 1040, 1055, 1070, 1085, 1100, 1115,
  1130, 1145, 26,   56,   86,   116,  146,  176,  206,  236,  266,  296,
  326,  356,  386,  416,  446,  476,  506,  536,  566,  596,  626,  656,
  686,  716,  746,  776,  806,  836,  866,  896,  925,  940,  955,  970,
  985,  1000, 1015, 1030, 1045, 1060, 1075, 1090, 1105, 1120, 1135, 1151,
  4,    34,   64,   94,   124,  154,  184,  214,  244,  274,  304,  334,
  364,  394,  424,  454,  484,  514,  544,  574,  604,  634,  664,  694,
  724,  754,  784,  814,  844,  874,  904,  929,  944,  959,  974,  989,
  1004, 1019, 1034, 1049, 1064, 1079, 1094, 1109, 1124, 1139, 14,   44,
  74,   104,  134,  164,  194,  224,  254,  284,  314,  344,  374,  404,
  434,  464,  494,  524,  554,  584,  614,  644,  674,  704,  734,  764,
  794,  824,  854,  884,  914,  934,  949,  964,  979,  994,  1009, 1024,
  1039, 1054, 1069, 1084, 1099, 1114, 1129, 1144, 24,   54,   84,   114,
  144,  174,  204,  234,  264,  294,  324,  354,  384,  414,  444,  474,
  504,  534,  564,  594,  624,  654,  684,  714,  744,  774,  804,  834,
  864,  894,  924,  939,  954,  969,  984,  999,  1014, 1029, 1044, 1059,
  1074, 1089, 1104, 1119, 1134, 2303, 19,   49,   79,   109,  139,  169,
  199,  229,  259,  289,  319,  349,  379,  409,  439,  469,  499,  529,
  559,  589,  619,  649,  679,  709,  739,  769,  799,  829,  859,  889,
  919,  2365, 1290, 2517, 1442, 2667, 1593, 2819, 1745, 2971, 1896, 3123,
  2048, 3275, 2199, 3425, 9,    39,   69,   99,   129,  159,  189,  219,
  249,  279,  309,  339,  369,  399,  429,  459,  489,  519,  549,  579,
  609,  639,  669,  699,  729,  759,  789,  819,  849,  879,  909,  1189,
  2415, 1340, 2567, 1492, 2719, 1644, 2869, 1795, 3021, 1947, 3173, 2098,
  3325, 2250, 29,   59,   89,   119,  149,  179,  209,  239,  269,  299,
  329,  359,  389,  419,  449,  479,  509,  539,  569,  599,  629,  659,
  689,  719,  749,  779,  809,  839,  869,  899,  2315, 1239, 2465, 1391,
  2617, 1543, 2769, 1694, 2921, 1846, 3073, 1997, 3223, 2149, 3375, 2300,
  12,   42,   72,   102,  132,  162,  192,  222,  252,  282,  312,  342,
  372,  402,  432,  462,  492,  522,  552,  582,  612,  642,  672,  702,
  732,  762,  792,  822,  852,  882,  912,  933,  948,  963,  978,  993,
  1008, 1023, 1038, 1053, 1068, 1083, 1098, 1113, 1128, 1143, 2,    32,
  62,   92,   122,  152,  182,  212,  242,  272,  302,  332,  362,  392,
  422,  452,  482,  512,  542,  572,  602,  632,  662,  692,  722,  752,
  782,  812,  842,  872,  902,  928,  943,  958,  973,  988,  1003, 1018,
  1033, 1048, 1063, 1078, 1093, 1108, 1123, 1138, 7,    37,   67,   97,
  127,  157,  187,  217,  247,  277,  307,  337,  367,  397,  427,  457,
  487,  517,  547,  577,  607,  637,  667,  697,  727,  757,  787,  817,
  847,  877,  907,  2335, 1260, 2485, 1411, 2637, 1563, 2789, 1714, 2941,
  1866, 3093, 2018, 3243, 2169, 3395, 22,   52,   82,   112,  142,  172,
  202,  232,  262,  292,  322,  352,  382,  412,  442,  472,  502,  532,
  562,  592,  622,  652,  682,  712,  742,  772,  802,  832,  862,  892,
  922,  938,  953,  968,  983,  998,  1013, 1028, 1043, 1058, 1073, 1088,
  1103, 1118, 1133, 1148, 27,   57,   87,   117,  147,  177,  207,  237,
  267,  297,  327,  357,  387,  417,  447,  477,  507,  537,  567,  597,
  627,  657,  687,  717,  747,  777,  807,  837,  867,  897,  1159, 2385,
  1310, 2537, 1462, 2689, 1613, 2839, 1765, 2991, 1916, 3143, 2068, 3295,
  2220, 3445, 17,   47,   77,   107,  137,  167,  197,  227,  257,  287,
  317,  347,  377,  407,  437,  467,  497,  527,  557,  587,  617,  647,
  677,  707,  737,  767,  797,  827,  857,  887,  917,  1209, 2435, 1361,
  2587, 1512, 2739, 1664, 2891, 1815, 3041, 1967, 3193, 2119, 3345, 2270
  },
  {
  0,    30,   60,   90,   120,  150,  180,  210,  240,  270,  300,  330,
  360,  390,  420,  450,  480,  510,  540,  570,  600,  630,  660,  690,
  720,  750,  780,  810,  840,  870,  900,  927,  942,  957,  972,  987,
  1002, 1017, 1032, 1047, 1062, 1077, 1092, 1107, 1122, 1137, 20,   50,
  80,   110,  140,  170,  200,  230,  260,  290,  320,  350,  380,  410,
  440,  470,  500,  530,  560,  590,  620,  650,  680,  710,  740,  770,
  800,  830,  860,  890,  920,  937,  952,  967,  982,  997,  1012, 1027,
  1042, 1057, 1072, 1087, 1102, 1117, 1132, 1147, 10,   40,   70,   100,
  130,  160,  190,  220,  250,  280,  310,  340,  370,  400,  430,  460,
  490,  520,  550,  580,  610,  640,  670,  700,  730,  760,  790,  820,
  850,  880,  910,  932,  947,  962,  977,  992,  1007, 1022, 1037, 1052,
  1067, 1082, 1097, 1112, 1127, 1142, 5,    35,   65,   95,   125,  155,
  185,  215,  245,  275,  305,  335,  365,  395,  425,  455,  485,  515,
  545,  575,  605,  635,  665,  695,  725,  755,  785,  815,  845,  875,
  905,  1181, 2408, 1333, 2559, 1485, 2711, 1637, 2862, 1787, 3014, 1939,
  3165, 2091, 3317, 2243, 15,   45,   75,   105,  135,  165,  195,  225,
  255,  285,  315,  345,  375,  405,  435,  465,  495,  525,  555,  585,
  615,  645,  675,  705,  735,  765,  795,  825,  855,  885,  915,  2357,
  1283, 2509, 1433, 2660, 1585, 2812, 1737, 2963, 1889, 3115, 2041, 3267,
  2191, 3418, 25,   55,   85,   115,  145,  175,  205,  235,  265,  295,
  325,  355,  385,  415,  445,  475,  505,  535,  565,  595,  625,  655,
  685,  715,  745,  775,  805,  835,  865,  895,  2307, 1231, 2458, 1383,
  2610, 1535, 2761, 1687, 2913, 1839, 3064, 1989, 3216, 2141, 3368, 2293,
  3,    33,   63,   93,   123,  153,  183,  213,  243,  273,  303,  333,
  363,  393,  423,  453,  483,  513,  543,  573,  603,  633,  663,  693,
  723,  753,  783,  813,  843,  873,  903,  2327, 1253, 2478, 1403, 2630,
  1555, 2781, 1707, 2933, 1859, 3085, 2009, 3236, 2161, 3388, 13,   43,
  73,   103,  133,  163,  193,  223,  253,  283,  313,  343,  373,  403,
  433,  463,  493,  523,  553,  583,  613,  643,  673,  703,  733,  763,
  793,  823,  853,  883,  913,  1201, 2428, 1353, 2579, 1505, 2731, 1657,
  2883, 1807, 3034, 1959, 3186, 2111, 3337, 2263, 23,   53,   83,   113,
  143,  173,  203,  233,  263,  293,  323,  353,  383,  413,  443,  473,
  503,  533,  563,  593,  623,  653,  683,  713,  743,  773,  803,  833,
  863,  893,  923,  2377, 1303, 2529, 1455, 2680, 1605, 2832, 1757, 2984,
  1909, 3135, 2061, 3287, 2213, 3438, 8,    38,   68,   98,   128,  158,
  188,  218,  248,  278,  308,  338,  368,  398,  428,  458,  488,  518,
  548,  578,  608,  638,  668,  698,  728,  758,  788,  818,  848,  878,
  908,  931,  946,  961,  976,  991,  1006, 1021, 1036, 1051, 1066, 1081,
  1096, 1111, 1126, 1141, 18,   48,   78,   108,  138,  168,  198,  228,
  258,  288,  318,  348,  378,  408,  438,  468,  498,  528,  558,  588,
  618,  648,  678,  708,  738,  768,  798,  828,  858,  888,  918,  936,
  951,  966,  981,  996,  1011, 1026, 1041, 1056, 1071, 1086, 1101, 1116,
  1131, 1146, 28,   58,   88,   118,  148,  178,  208,  238,  268,  298,
  328,  358,  388,  418,  448,  478,  508,  538,  568,  598,  628,  658,
  688,  718,  748,  778,  808,  838,  868,  898,  926,  941,  956,  971,
  986,  1001, 1016, 1031, 1046, 1061, 1076, 1091, 1106, 1121, 1136, 3454,
  1,    31,   61,   91,   121,  151,  181,  211,  241,  271,  301,  331,
  361,  391,  421,  451,  481,  511,  541,  571,  601,  631,  661,  691,
  721,  751,  781,  811,  841,  871,  901,  1171, 2397, 1323, 2549, 1475,
  2701, 1625, 2852, 1777, 3004, 1929, 3155, 2081, 3307, 2233, 11,   41,
  71,   101,  131,  161,  191,  221,  251,  281,  311,  341,  371,  401,
  431,  461,  491,  521,  551,  581,  611,  641,  671,  701,  731,  761,
  791,  821,  851,  881,  911,  2347, 1273, 2499, 1423, 2650, 1575, 2802,
  1727, 2953, 1879, 3105, 2031, 3256, 2181, 3408, 21,   51,   81,   111,
  141,  171,  201,  231,  261,  291,  321,  351,  381,  411,  441,  471,
  501,  531,  561,  591,  621,  651,  681,  711,  741,  771,  801,  831,
  861,  891,  921,  1221, 2448, 1373, 2600, 1525, 2751, 1677, 2903, 1829,
  3054, 1979, 3206, 2131, 3357, 2283, 6,    36,   66,   96,   126,  156,
  186,  216,  246,  276,  306,  336,  366,  396,  426,  456,  486,  516,
  546,  576,  606,  636,  666,  696,  726,  756,  786,  816,  846,  876,
  906,  930,  945,  960,  975,  990,  1005, 1020, 1035, 1050, 1065, 1080,
  1095, 1110, 1125, 1140, 16,   46,   76,   106,  136,  166,  196,  226,
  256,  286,  316,  346,  376,  406,  436,  466,  496,  526,  556,  586,
  616,  646,  676,  706,  736,  766,  796,  826,  856,  886,  916,  935,
  950,  965,  980,  995,  1010, 1025, 1040, 1055, 1070, 1085, 1100, 1115,
  1130, 1145, 26,   56,   86,   116,  146,  176,  206,  236,  266,  296,
  326,  356,  386,  416,  446,  476,  506,  536,  566,  596,  626,  656,
  686,  716,  746,  776,  806,  836,  866,  896,  925,  940,  955,  970,
  985,  1000, 1015, 1030, 1045, 1060, 1075, 1090, 1105, 1120, 1135, 1151,
  4,    34,   64,   94,   124,  154,  184,  214,  244,  274,  304,  334,
  364,  394,  424,  454,  484,  514,  544,  574,  604,  634,  664,  694,
  724,  754,  784,  814,  844,  874,  904,  929,  944,  959,  974,  989,
  1004, 1019, 1034, 1049, 1064, 1079, 1094, 1109, 1124, 1139, 14,   44,
  74,   104,  134,  164,  194,  224,  254,  284,  314,  344,  374,  404,
  434,  464,  494,  524,  554,  584,  614,  644,  674,  704,  734,  764,
  794,  824,  854,  884,  914,  934,  949,  964,  979,  994,  1009, 1024,
  1039, 1054, 1069, 1084, 1099, 1114, 1129, 1144, 24,   54,   84,   114,
  144,  174,  204,  234,  264,  294,  324,  354,  384,  414,  444,  474,
  504,  534,  564,  594,  624,  654,  684,  714,  744,  774,  804,  834,
  864,  894,  924,  939,  954,  969,  984,  999,  1014, 1029, 1044, 1059,
  1074, 1089, 1104, 1119, 1134, 2303, 19,   49,   79,   109,  139,  169,
  199,  229,  259,  289,  319,  349,  379,  409,  439,  469,  499,  529,
  559,  589,  619,  649,  679,  709,  739,  769,  799,  829,  859,  889,
  919,  2367, 1293, 2519, 1445, 2670, 1595, 2822, 1747, 2973, 1899, 3125,
  2051, 3277, 2201, 3428, 9,    39,   69,   99,   129,  159,  189,  219,
  249,  279,  309,  339,  369,  399,  429,  459,  489,  519,  549,  579,
  609,  639,  669,  699,  729,  759,  789,  819,  849,  879,  909,  1191,
  2418, 1343, 2569, 1495, 2721, 1647, 2872, 1797, 3024, 1949, 3176, 2101,
  3327, 2253, 29,   59,   89,   119,  149,  179,  209,  239,  269,  299,
  329,  359,  389,  419,  449,  479,  509,  539,  569,  599,  629,  659,
  689,  719,  749,  779,  809,  839,  869,  899,  2317, 1241, 2468, 1393,
  2620, 1545, 2771, 1697, 2923, 1849, 3075, 1999, 3226, 2151, 3378, 1150,
  12,   42,   72,   102,  132,  162,  192,  222,  252,  282,  312,  342,
  372,  402,  432,  462,  492,  522,  552,  582,  612,  642,  672,  702,
  732,  762,  792,  822,  852,  882,  912,  933,  948,  963,  978,  993,
  1008, 1023, 1038, 1053, 1068, 1083, 1098, 1113, 1128, 1143, 2,    32,
  62,   92,   122,  152,  182,  212,  242,  272,  302,  332,  362,  392,
  422,  452,  482,  512,  542,  572,  602,  632,  662,  692,  722,  752,
  782,  812,  842,  872,  902,  928,  943,  958,  973,  988,  1003, 1018,
  1033, 1048, 1063, 1078, 1093, 1108, 1123, 1138, 7,    37,   67,   97,
  127,  157,  187,  217,  247,  277,  307,  337,  367,  397,  427,  457,
  487,  517,  547,  577,  607,  637,  667,  697,  727,  757,  787,  817,
  847,  877,  907,  2337, 1263, 2488, 1413, 2640, 1565, 2792, 1717, 2943,
  1869, 3095, 2021, 3246, 2171, 3398, 22,   52,   82,   112,  142,  172,
  202,  232,  262,  292,  322,  352,  382,  412,  442,  472,  502,  532,
  562,  592,  622,  652,  682,  712,  742,  772,  802,  832,  862,  892,
  922,  938,  953,  968,  983,  998,  1013, 1028, 1043, 1058, 1073, 1088,
  1103, 1118, 1133, 1148, 27,   57,   87,   117,  147,  177,  207,  237,
  267,  297,  327,  357,  387,  417,  447,  477,  507,  537,  567,  597,
  627,  657,  687,  717,  747,  777,  807,  837,  867,  897,  1161, 2387,
  1313, 2539, 1465, 2691, 1615, 2842, 1767, 2994, 1919, 3145, 2071, 3297,
  2223, 3448, 17,   47,   77,   107,  137,  167,  197,  227,  257,  287,
  317,  347,  377,  407,  437,  467,  497,  527,  557,  587,  617,  647,
  677,  707,  737,  767,  797,  827,  857,  887,  917,  1211, 2438, 1363,
  2589, 1515, 2741, 1667, 2893, 1817, 3044, 1969, 3196, 2121, 3347, 2273
  },
  {
  0,    30,   60,   90,   120,  150,  180,  210,  240,  270,  300,  330,
  360,  390,  420,  450,  480,  510,  540,  570,  600,  630,  660,  690,
  720,  750,  780,  810,  840,  870,  900,  927,  942,  957,  972,  987,
  1002, 1017, 1032, 1047, 1062, 1077, 1092, 1107, 1122, 1137, 20,   50,
  80,   110,  140,  170,  200,  230,  260,  290,  320,  350,  380,  410,
  440,  470,  500,  530,  560,  590,  620,  650,  680,  710,  740,  770,
  800,  830,  860,  890,  920,  937,  952,  967,  982,  997,  1012, 1027,
  1042, 1057, 1072, 1087, 1102, 1117, 1132, 1147, 10,   40,   70,   100,
  130,  160,  190,  220,  250,  280,  310,  340,  370,  400,  430,  460,
  490,  520,  550,  580,  610,  640,  670,  700,  730,  760,  790,  820,
  850,  880,  910,  932,  947,  962,  977,  992,  1007, 1022, 1037, 1052,
  1067, 1082, 1097, 1112, 1127, 1142, 5,    35,   65,   95,   125,  155,
  185,  215,  245,  275,  305,  335,  365,  395,  425,  455,  485,  515,
  545,  575,  605,  635,  665,  695,  725,  755,  785,  815,  845,  875,
  905,  1174, 2410, 1325, 2562, 1477, 2714, 1628, 2864, 1780, 3016, 1932,
  3168, 2083, 3320, 2235, 15,   45,   75,   105,  135,  165,  195,  225,
  255,  285,  315,  345,  375,  405,  435,  465,  495,  525,  555,  585,
  615,  645,  675,  705,  735,  765,  795,  825,  855,  885,  915,  2360,
  1275, 2512, 1426, 2662, 1578, 2814, 1730, 2966, 1881, 3118, 2033, 3270,
  2184, 3420, 25,   55,   85,   115,  145,  175,  205,  235,  265,  295,
  325,  355,  385,  415,  445,  475,  505,  535,  565,  595,  625,  655,
  685,  715,  745,  775,  805,  835,  865,  895,  2310, 1224, 2460, 1376,
  2612, 1527, 2764, 1679, 2916, 1831, 3066, 1982, 3218, 2134, 3370, 2285,
  3,    33,   63,   93,   123,  153,  183,  213,  243,  273,  303,  333,
  363,  393,  423,  453,  483,  513,  543,  573,  603,  633,  663,  693,
  723,  753,  783,  813,  843,  873,  903,  2330, 1244, 2480, 1396, 2632,
  1548, 2784, 1699, 2936, 1851, 3088, 2002, 3238, 2154, 3390, 13,   43,
  73,   103,  133,  163,  193,  223,  253,  283,  313,  343,  373,  403,
  433,  463,  493,  523,  553,  583,  613,  643,  673,  703,  733,  763,
  793,  823,  853,  883,  913,  1194, 2430, 1346, 2582, 1497, 2734, 1649,
  2886, 1800, 3036, 1952, 3188, 2103, 3340, 2255, 23,   53,   83,   113,
  143,  173,  203,  233,  263,  293,  323,  353,  383,  413,  443,  473,
  503,  533,  563,  593,  623,  653,  683,  713,  743,  773,  803,  833,
  863,  893,  923,  2380, 1295, 2532, 1447, 2682, 1598, 2834, 1750, 2986,
  1901, 3138, 2053, 3290, 2204, 3440, 8,    38,   68,   98,   128,  158,
  188,  218,  248,  278,  308,  338,  368,  398,  428,  458,  488,  518,
  548,  578,  608,  638,  668,  698,  728,  758,  788,  818,  848,  878,
  908,  931,  946,  961,  976,  991,  1006, 1021, 1036, 1051, 1066, 1081,
  1096, 1111, 1126, 1141, 18,   48,   78,   108,  138,  168,  198,  228,
  258,  288,  318,  348,  378,  408,  438,  468,  498,  528,  558,  588,
  618,  648,  678,  708,  738,  768,  798,  828,  858,  888,  918,  936,
  951,  966,  981,  996,  1011, 1026, 1041, 1056, 1071, 1086, 1101, 1116,
  1131, 1146, 28,   58,   88,   118,  148,  178,  208,  238,  268,  298,
  328,  358,  388,  418,  448,  478,  508,  538,  568,  598,  628,  658,
  688,  718,  748,  778,  808,  838,  868,  898,  926,  941,  956,  971,
  986,  1001, 1016, 1031, 1046, 1061, 1076, 1091, 1106, 1121, 1136, 3454,
  1,    31,   61,   91,   121,  151,  181,  211,  241,  271,  301,  331,
  361,  391,  421,  451,  481,  511,  541,  571,  601,  631,  661,  691,
  721,  751,  781,  811,  841,  871,  901,  1164, 2400, 1315, 2552, 1467,
  2704, 1618, 2854, 1770, 3006, 1922, 3158, 2073, 3310, 2225, 11,   41,
  71,   101,  131,  161,  191,  221,  251,  281,  311,  341,  371,  401,
  431,  461,  491,  521,  551,  581,  611,  641,  671,  701,  731,  761,
  791,  821,  851,  881,  911,  2350, 1265, 2502, 1416, 2652, 1568, 2804,
  1719, 2956, 1871, 3108, 2023, 3258, 2174, 3410, 21,   51,   81,   111,
  141,  171,  201,  231,  261,  291,  321,  351,  381,  411,  441,  471,
  501,  531,  561,  591,  621,  651,  681,  711,  741,  771,  801,  831,
  861,  891,  921,  1214, 2450, 1366, 2602, 1517, 2754, 1669, 2906, 1820,
  3056, 1972, 3208, 2124, 3360, 2275, 6,    36,   66,   96,   126,  156,
  186,  216,  246,  276,  306,  336,  366,  396,  426,  456,  486,  516,
  546,  576,  606,  636,  666,  696,  726,  756,  786,  816,  846,  876,
  906,  930,  945,  960,  975,  990,  1005, 1020, 1035, 1050, 1065, 1080,
  1095, 1110, 1125, 1140, 16,   46,   76,   106,  136,  166,  196,  226,
  256,  286,  316,  346,  376,  406,  436,  466,  496,  526,  556,  586,
  616,  646,  676,  706,  736,  766,  796,  826,  856,  886,  916,  935,
  950,  965,  980,  995,  1010, 1025, 1040, 1055, 1070, 1085, 1100, 1115,
  1130, 1145, 26,   56,   86,   116,  146,  176,  206,  236,  266,  296,
  326,  356,  386,  416,  446,  476,  506,  536,  566,  596,  626,  656,
  686,  716,  746,  776,  806,  836,  866,  896,  925,  940,  955,  970,
  985,  1000, 1015, 1030, 1045, 1060, 1075, 1090, 1105, 1120, 1135, 1151,
  4,    34,   64,   94,   124,  154,  184,  214,  244,  274,  304,  334,
  364,  394,  424,  454,  484,  514,  544,  574,  604,  634,  664,  694,
  724,  754,  784,  814,  844,  874,  904,  929,  944,  959,  974,  989,
  1004, 1019, 1034, 1049, 1064, 1079, 1094, 1109, 1124, 1139, 14,   44,
  74,   104,  134,  164,  194,  224,  254,  284,  314,  344,  374,  404,
  434,  464,  494,  524,  554,  584,  614,  644,  674,  704,  734,  764,
  794,  824,  854,  884,  914,  934,  949,  964,  979,  994,  1009, 1024,
  1039, 1054, 1069, 1084, 1099, 1114, 1129, 1144, 24,   54,   84,   114,
  144,  174,  204,  234,  264,  294,  324,  354,  384,  414,  444,  474,
  504,  534,  564,  594,  624,  654,  684,  714,  744,  774,  804,  834,
  864,  894,  924,  939,  954,  969,  984,  999,  1014, 1029, 1044, 1059,
  1074, 1089, 1104, 1119, 1134, 2303, 19,   49,   79,   109,  139,  169,
  199,  229,  259,  289,  319,  349,  379,  409,  439,  469,  499,  529,
  559,  589,  619,  649,  679,  709,  739,  769,  799,  829,  859,  889,
  919,  2370, 1285, 2522, 1436, 2672, 1588, 2824, 1740, 2976, 1891, 3128,
  2043, 3280, 2194, 3430, 9,    39,   69,   99,   129,  159,  189,  219,
  249,  279,  309,  339,  369,  399,  429,  459,  489,  519,  549,  579,
  609,  639,  669,  699,  729,  759,  789,  819,  849,  879,  909,  1184,
  2420, 1335, 2572, 1487, 2724, 1639, 2874, 1790, 3026, 1942, 3178, 2093,
  3330, 2245, 29,   59,   89,   119,  149,  179,  209,  239,  269,  299,
  329,  359,  389,  419,  449,  479,  509,  539,  569,  599,  629,  659,
  689,  719,  749,  779,  809,  839,  869,  899,  2320, 1234, 2470, 1386,
  2622, 1538, 2774, 1689, 2926, 1841, 3078, 1992, 3228, 2144, 3380, 2295,
  12,   42,   72,   102,  132,  162,  192,  222,  252,  282,  312,  342,
  372,  402,  432,  462,  492,  522,  552,  582,  612,  642,  672,  702,
  732,  762,  792,  822,  852,  882,  912,  933,  948,  963,  978,  993,
  1008, 1023, 1038, 1053, 1068, 1083, 1098, 1113, 1128, 1143, 2,    32,
  62,   92,   122,  152,  182,  212,  242,  272,  302,  332,  362,  392,
  422,  452,  482,  512,  542,  572,  602,  632,  662,  692,  722,  752,
  782,  812,  842,  872,  902,  928,  943,  958,  973,  988,  1003, 1018,
  1033, 1048, 1063, 1078, 1093, 1108, 1123, 1138, 7,    37,   67,   97,
  127,  157,  187,  217,  247,  277,  307,  337,  367,  397,  427,  457,
  487,  517,  547,  577,  607,  637,  667,  697,  727,  757,  787,  817,
  847,  877,  907,  2340, 1255, 2490, 1406, 2642, 1558, 2794, 1709, 2946,
  1861, 3098, 2012, 3248, 2164, 3400, 22,   52,   82,   112,  142,  172,
  202,  232,  262,  292,  322,  352,  382,  412,  442,  472,  502,  532,
  562,  592,  622,  652,  682,  712,  742,  772,  802,  832,  862,  892,
  922,  938,  953,  968,  983,  998,  1013, 1028, 1043, 1058, 1073, 1088,
  1103, 1118, 1133, 1148, 27,   57,   87,   117,  147,  177,  207,  237,
  267,  297,  327,  357,  387,  417,  447,  477,  507,  537,  567,  597,
  627,  657,  687,  717,  747,  777,  807,  837,  867,  897,  1154, 2390,
  1305, 2542, 1457, 2694, 1608, 2844, 1760, 2996, 1911, 3148, 2063, 3300,
  2215, 3450, 17,   47,   77,   107,  137,  167,  197,  227,  257,  287,
  317,  347,  377,  407,  437,  467,  497,  527,  557,  587,  617,  647,
  677,  707,  737,  767,  797,  827,  857,  887,  917,  1204, 2440, 1356,
  2592, 1507, 2744, 1659, 2896, 1810, 3046, 1962, 3198, 2114, 3350, 2265
  },
  {
  0,    30,   60,   90,   120,  150,  180,  210,  240,  270,  300,  330,
  360,  390,  420,  450,  480,  510,  540,  570,  600,  630,  660,  690,
  720,  750,  780,  810,  840,  870,  900,  927,  942,  957,  972,  987,
  1002, 1017, 1032, 1047, 1062, 1077, 1092, 1107, 1122, 1137, 20,   50,
  80,   110,  140,  170,  200,  230,  260,  290,  320,  350,  380,  410,
  440,  470,  500,  530,  560,  590,  620,  650,  680,  710,  740,  770,
  800,  830,  860,  890,  920,  937,  952,  967,  982,  997,  1012, 1027,
  1042, 1057, 1072, 1087, 1102, 1117, 1132, 1147, 10,   40,   70,   100,
  130,  160,  190,  220,  250,  280,  310,  340,  370,  400,  430,  460,
  490,  520,  550,  580,  610,  640,  670,  700,  730,  760,  790,  820,
  850,  880,  910,  932,  947,  962,  977,  992,  1007, 1022, 1037, 1052,
  1067, 1082, 1097, 1112, 1127, 1142, 5,    35,   65,   95,   125,  155,
  185,  215,  245,  275,  305,  335,  365,  395,  425,  455,  485,  515,
  545,  575,  605,  635,  665,  695,  725,  755,  785,  815,  845,  875,
  905,  1176, 2413, 1328, 2564, 1480, 2716, 1630, 2867, 1782, 3019, 1934,
  3171, 2086, 3322, 2238, 15,   45,   75,   105,  135,  165,  195,  225,
  255,  285,  315,  345,  375,  405,  435,  465,  495,  525,  555,  585,
  615,  645,  675,  705,  735,  765,  795,  825,  855,  885,  915,  2362,
  1278, 2514, 1428, 2665, 1580, 2817, 1732, 2968, 1884, 3120, 2036, 3272,
  2186, 3423, 25,   55,   85,   115,  145,  175,  205,  235,  265,  295,
  325,  355,  385,  415,  445,  475,  505,  535,  565,  595,  625,  655,
  685,  715,  745,  775,  805,  835,  865,  895,  2312, 1226, 2463, 1378,
  2615, 1530, 2766, 1682, 2918, 1834, 3069, 1984, 3221, 2136, 3373, 2288,
  3,    33,   63,   93,   123,  153,  183,  213,  243,  273,  303,  333,
  363,  393,  423,  453,  483,  513,  543,  573,  603,  633,  663,  693,
  723,  753,  783,  813,  843,  873,  903,  2332, 1246, 2483, 1398, 2635,
  1550, 2787, 1702, 2938, 1854, 3090, 2004, 3241, 2156, 3393, 13,   43,
  73,   103,  133,  163,  193,  223,  253,  283,  313,  343,  373,  403,
  433,  463,  493,  523,  553,  583,  613,  643,  673,  703,  733,  763,
  793,  823,  853,  883,  913,  1196, 2433, 1348, 2584, 1500, 2736, 1652,
  2888, 1802, 3039, 1954, 3191, 2106, 3342, 2258, 23,   53,   83,   113,
  143,  173,  203,  233,  263,  293,  323,  353,  383,  413,  443,  473,
  503,  533,  563,  593,  623,  653,  683,  713,  743,  773,  803,  833,
  863,  893,  923,  2382, 1298, 2534, 1450, 2685, 1600, 2837, 1752, 2989,
  1904, 3140, 2056, 3292, 2206, 3443, 8,    38,   68,   98,   128,  158,
  188,  218,  248,  278,  308,  338,  368,  398,  428,  458,  488,  518,
  548,  578,  608,  638,  668,  698,  728,  758,  788,  818,  848,  878,
  908,  931,  946,  961,  976,  991,  1006, 1021, 1036, 1051, 1066, 1081,
  1096, 1111, 1126, 1141, 18,   48,   78,   108,  138,  168,  198,  228,
  258,  288,  318,  348,  378,  408,  438,  468,  498,  528,  558,  588,
  618,  648,  678,  708,  738,  768,  798,  828,  858,  888,  918,  936,
  951,  966,  981,  996,  1011, 1026, 1041, 1056, 1071, 1086, 1101, 1116,
  1131, 1146, 28,   58,   88,   118,  148,  178,  208,  238,  268,  298,
  328,  358,  388,  418,  448,  478,  508,  538,  568,  598,  628,  658,
  688,  718,  748,  778,  808,  838,  868,  898,  926,  941,  956,  971,
  986,  1001, 1016, 1031, 1046, 1061, 1076, 1091, 1106, 1121, 1136, 3454,
  1,    31,   61,   91,   121,  151,  181,  211,  241,  271,  301,  331,
  361,  391,  421,  451,  481,  511,  541,  571,  601,  631,  661,  691,
  721,  751,  781,  811,  841,  871,  901,  1166, 2403, 1318, 2554, 1470,
  2706, 1620, 2857, 1772, 3009, 1924, 3160, 2076, 3312, 2228, 11,   41,
  71,   101,  131,  161,  191,  221,  251,  281,  311,  341,  371,  401,
  431,  461,  491,  521,  551,  581,  611,  641,  671,  701,  731,  761,
  791,  821,  851,  881,  911,  2352, 1268, 2504, 1418, 2655, 1570, 2807,
  1722, 2958, 1874, 3110, 2026, 3261, 2176, 3413, 21,   51,   81,   111,
  141,  171,  201,  231,  261,  291,  321,  351,  381,  411,  441,  471,
  501,  531,  561,  591,  621,  651,  681,  711,  741,  771,  801,  831,
  861,  891,  921,  1216, 2453, 1368, 2605, 1520, 2756, 1672, 2908, 1822,
  3059, 1974, 3211, 2126, 3363, 2278, 6,    36,   66,   96,   126,  156,
  186,  216,  246,  276,  306,  336,  366,  396,  426,  456,  486,  516,
  546,  576,  606,  636,  666,  696,  726,  756,  786,  816,  846,  876,
  906,  930,  945,  960,  975,  990,  1005, 1020, 1035, 1050, 1065, 1080,
  1095, 1110, 1125, 1140, 16,   46,   76,   106,  136,  166,  196,  226,
  256,  286,  316,  346,  376,  406,  436,  466,  496,  526,  556,  586,
  616,  646,  676,  706,  736,  766,  796,  826,  856,  886,  916,  935,
  950,  965,  980,  995,  1010, 1025, 1040, 1055, 1070, 1085, 1100, 1115,
  1130, 1145, 26,   56,   86,   116,  146,  176,  206,  236,  266,  296,
  326,  356,  386,  416,  446,  476,  506,  536,  566,  596,  626,  656,
  686,  716,  746,  776,  806,  836,  866,  896,  925,  940,  955,  970,
  985,  1000, 1015, 1030, 1045, 1060, 1075, 1090, 1105, 1120, 1135, 1151,
  4,    34,   64,   94,   124,  154,  184,  214,  244,  274,  304,  334,
  364,  394,  424,  454,  484,  514,  544,  574,  604,  634,  664,  694,
  724,  754,  784,  814,  844,  874,  904,  929,  944,  959,  974,  989,
  1004, 1019, 1034, 1049, 1064, 1079, 1094, 1109, 1124, 1139, 14,   44,
  74,   104,  134,  164,  194,  224,  254,  284,  314,  344,  374,  404,
  434,  464,  494,  524,  554,  584,  614,  644,  674,  704,  734,  764,
  794,  824,  854,  884,  914,  934,  949,  964,  979,  994,  1009, 1024,
  1039, 1054, 1069, 1084, 1099, 1114, 1129, 1144, 24,   54,   84,   114,
  144,  174,  204,  234,  264,  294,  324,  354,  384,  414,  444,  474,
  504,  534,  564,  594,  624,  654,  684,  714,  744,  774,  804,  834,
  864,  894,  924,  939,  954,  969,  984,  999,  1014, 1029, 1044, 1059,
  1074, 1089, 1104, 1119, 1134, 2303, 19,   49,   79,   109,  139,  169,
  199,  229,  259,  289,  319,  349,  379,  409,  439,  469,  499,  529,
  559,  589,  619,  649,  679,  709,  739,  769,  799,  829,  859,  889,
  919,  2372, 1288, 2524, 1438, 2675, 1590, 2827, 1742, 2979, 1894, 3130,
  2046, 3282, 2196, 3433, 9,    39,   69,   99,   129,  159,  189,  219,
  249,  279,  309,  339,  369,  399,  429,  459,  489,  519,  549,  579,
  609,  639,  669,  699,  729,  759,  789,  819,  849,  879,  909,  1186,
  2423, 1338, 2574, 1490, 2726, 1642, 2877, 1792, 3029, 1944, 3181, 2096,
  3332, 2248, 29,   59,   89,   119,  149,  179,  209,  239,  269,  299,
  329,  359,  389,  419,  449,  479,  509,  539,  569,  599,  629,  659,
  689,  719,  749,  779,  809,  839,  869,  899,  2322, 1236, 2473, 1388,
  2625, 1540, 2776, 1692, 2928, 1844, 3080, 1994, 3231, 2146, 3383, 2298,
  12,   42,   72,   102,  132,  162,  192,  222,  252,  282,  312,  342,
  372,  402,  432,  462,  492,  522,  552,  582,  612,  642,  672,  702,
  732,  762,  792,  822,  852,  882,  912,  933,  948,  963,  978,  993,
  1008, 1023, 1038, 1053, 1068, 1083, 1098, 1113, 1128, 1143, 2,    32,
  62,   92,   122,  152,  182,  212,  242,  272,  302,  332,  362,  392,
  422,  452,  482,  512,  542,  572,  602,  632,  662,  692,  722,  752,
  782,  812,  842,  872,  902,  928,  943,  958,  973,  988,  1003, 1018,
  1033, 1048, 1063, 1078, 1093, 1108, 1123, 1138, 7,    37,   67,   97,
  127,  157,  187,  217,  247,  277,  307,  337,  367,  397,  427,  457,
  487,  517,  547,  577,  607,  637,  667,  697,  727,  757,  787,  817,
  847,  877,  907,  2342, 1258, 2493, 1408, 2645, 1560, 2797, 1712, 2948,
  1864, 3100, 2014, 3251, 2166, 3403, 22,   52,   82,   112,  142,  172,
  202,  232,  262,  292,  322,  352,  382,  412,  442,  472,  502,  532,
  562,  592,  622,  652,  682,  712,  742,  772,  802,  832,  862,  892,
  922,  938,  953,  968,  983,  998,  1013, 1028, 1043, 1058, 1073, 1088,
  1103, 1118, 1133, 1148, 27,   57,   87,   117,  147,  177,  207,  237,
  267,  297,  327,  357,  387,  417,  447,  477,  507,  537,  567,  597,
  627,  657,  687,  717,  747,  777,  807,  837,  867,  897,  1156, 2392,
  1308, 2544, 1460, 2696, 1610, 2847, 1762, 2999, 1914, 3150, 2066, 3302,
  2218, 1149, 17,   47,   77,   107,  137,  167,  197,  227,  257,  287,
  317,  347,  377,  407,  437,  467,  497,  527,  557,  587,  617,  647,
  677,  707,  737,  767,  797,  827,  857,  887,  917,  1206, 2443, 1358,
  2595, 1510, 2746, 1662, 2898, 1812, 3049, 1964, 3201, 2116, 3352, 2268
  }
};
